/******/ (() => { // webpackBootstrap
/******/ 	"use strict";
var __webpack_exports__ = {};

// UNUSED EXPORTS: ManagerRegistry

;// CONCATENATED MODULE: ./src/extension/managers/badge/BadgeManager.ts
class BadgeManager {
    static async getStorageData(keys) {
        return chrome.storage.local.get(keys);
    }
    static async updateBadge() {
        try {
            const count = await this.countQualifyingAccounts();
            if (count === 0) {
                await chrome.action.setBadgeText({ text: "" });
                await chrome.action.setBadgeBackgroundColor({ color: [0, 0, 0, 0] });
            }
            else {
                await chrome.action.setBadgeText({ text: count.toString() });
                await chrome.action.setBadgeBackgroundColor({ color: "#FF8080" });
            }
        }
        catch (error) {
            console.error("Error updating badge:", error);
        }
    }
    static async countQualifyingAccounts() {
        try {
            // Get stored accounts first to ensure we only check valid accounts
            const result = await this.getStorageData("storedAccounts");
            const storedAccounts = result?.storedAccounts;
            if (!storedAccounts || Object.keys(storedAccounts).length === 0) {
                return 0;
            }
            const accountIds = Object.keys(storedAccounts);
            let count = 0;
            // Build keys to fetch all at once
            const keysToFetch = accountIds.reduce((acc, id) => {
                acc.push(`isActive_${id}`, `failCount_for_${id}`, `paused_${id}`, `status_${id}`, `collection_status_${id}`);
                return acc;
            }, []);
            // Fetch all account data in one call
            const accountData = await this.getStorageData(keysToFetch);
            // Count qualifying accounts
            for (const id of accountIds) {
                const isActive = accountData[`isActive_${id}`];
                const failCount = accountData[`failCount_for_${id}`] || 0;
                const isPaused = accountData[`paused_${id}`];
                // Account qualifies if:
                // 1. It is active
                // 2. Has failures OR is in a failed state OR collection failed
                // 3. Is not paused
                if (isActive && !isPaused && failCount > 0) {
                    count++;
                }
            }
            return count;
        }
        catch (error) {
            console.error("Error counting qualifying accounts:", error);
            return 0;
        }
    }
    static initialize() {
        // Listen for relevant storage changes
        chrome.storage.onChanged.addListener((changes, namespace) => {
            if (namespace !== "local")
                return;
            const shouldUpdate = Object.keys(changes).some((key) => key.startsWith("isActive_") ||
                key.startsWith("failCount_for_") ||
                key.startsWith("paused_") ||
                key.startsWith("status_") ||
                key.startsWith("collection_status_") ||
                key === "storedAccounts");
            if (shouldUpdate) {
                this.updateBadge();
            }
        });
    }
}

;// CONCATENATED MODULE: ./src/extension/managers/alarm/AlarmTypes.ts
/**
 * Constants and types for the alarm management system
 */
const ALARM_TYPES = {
    COLLECTION: "collectAlarm_",
    HOURLY_CHECK: "hourly_check",
    RESOLVE: "resolve_alarm",
    KEEPALIVE: "keepalive",
    BALANCE_LOGGER: "balanceLogger",
};
// Delay constants
const RANDOM_DELAY_MIN = 6; // Minimum random delay in minutes
const RANDOM_DELAY_MAX = 17; // Maximum random delay in minutes
const BASE_RETRY_DELAY = 18; // Base retry delay in minutes
const MAX_RETRY_DELAY = 240; // Maximum retry delay in minutes

;// CONCATENATED MODULE: ./src/extension/managers/alarm/AlarmHelpers.ts
/**
 * Helper functions for alarm calculations and determinations
 */

/**
 * Storage key patterns for alarm-related data
 */
const STORAGE_KEYS = {
    COLLECTION_STATUS: (id) => `collection_status_${id}`,
    FAIL_COUNT: (id) => `failCount_for_${id}`,
    TIMESTAMP: (id) => `timestamp_for_${id}`,
    WAIT_PERIOD: (id) => `waitPeriod_for_${id}`,
    IS_ACTIVE: (id) => `isActive_${id}`,
    PAUSED: (id) => `paused_${id}`,
    MANUAL_ALARM: (id) => `manualAlarmSet_${id}`,
    ALARM_STATUS: (id) => `alarm_status_${id}`,
    COLLECTION_LOCK: (id) => `collection_lock_${id}`,
    ERROR_MESSAGE: (id) => `error_message_${id}`,
    VALUE_FOR: (id) => `value_for_${id}`,
    AUTH_METHOD: (id) => `authMethod_${id}`,
};
/**
 * Calculate retry delay with exponential backoff
 * @param failCount The number of consecutive failures
 * @returns Delay in milliseconds
 */
function getRetryDelay(failCount) {
    const baseDelay = BASE_RETRY_DELAY;
    const maxDelay = MAX_RETRY_DELAY;
    const exponentialDelay = baseDelay * Math.pow(2, failCount - 1);
    const jitter = (Math.random() - 0.5) * 5; // Add/subtract up to 2.5 minutes
    const delayMinutes = Math.min(Math.max(exponentialDelay + jitter, baseDelay), maxDelay);
    return delayMinutes * 60 * 1000; // Convert to milliseconds
}
/**
 * Get a random delay within the configured range
 * @returns Delay in milliseconds
 */
function getRandomDelay() {
    // Calculate random minutes between min and max (inclusive)
    const minutes = Math.random() * (RANDOM_DELAY_MAX - RANDOM_DELAY_MIN) + RANDOM_DELAY_MIN;
    // Use decimal part for seconds to add more randomness
    const fullMinutes = Math.floor(minutes);
    const seconds = Math.floor((minutes - fullMinutes) * 60);
    // Convert to milliseconds
    const delayMs = (fullMinutes * 60 + seconds) * 1000;
    // Log the calculated delay for debugging
    // console.log(
    //   `🎲 Generated random delay: ${fullMinutes}m ${seconds}s (${delayMs}ms)`
    // );
    return delayMs;
}
/**
 * Calculate the next trigger time based on the last collection time and wait period
 * @param timestamp The last collection timestamp (can be null)
 * @param waitPeriod The wait period in hours
 * @returns The next trigger time in milliseconds, or null if no timestamp
 */
function calculateNextTriggerTime(timestamp, waitPeriod) {
    // If no timestamp, return null (caller will handle this case)
    if (timestamp === null || timestamp === undefined) {
        console.log("No timestamp provided for next trigger calculation");
        return null;
    }
    // Convert string timestamp to number if needed
    let lastCollectionTime;
    if (typeof timestamp === "string") {
        // Check if it's an ISO string (contains T and possibly Z)
        if (timestamp.includes("T")) {
            try {
                lastCollectionTime = new Date(timestamp).getTime();
            }
            catch (error) {
                console.error(`Failed to parse ISO timestamp: ${timestamp}`, error);
                return null;
            }
        }
        else {
            // It's a numeric string
            try {
                lastCollectionTime = parseInt(timestamp, 10);
                // console.log(
                //   `Converted numeric timestamp to ${new Date(
                //     lastCollectionTime
                //   ).toLocaleString()}`
                // );
            }
            catch (error) {
                console.error(`Failed to parse numeric timestamp: ${timestamp}`, error);
                return null;
            }
        }
    }
    else {
        lastCollectionTime = timestamp;
    }
    // Validate the timestamp is a valid number and makes sense as a date
    if (isNaN(lastCollectionTime) || lastCollectionTime <= 0) {
        console.warn("⚠️ Invalid timestamp value:", timestamp);
        return null;
    }
    // Additional validation: check if timestamp is in a reasonable range
    // (between 2020 and 1 year from now)
    const minValidTime = new Date("2020-01-01").getTime();
    const maxValidTime = Date.now() + 365 * 24 * 60 * 60 * 1000; // 1 year from now
    if (lastCollectionTime < minValidTime || lastCollectionTime > maxValidTime) {
        console.warn(`⚠️ Timestamp out of reasonable range: ${new Date(lastCollectionTime).toLocaleString()}`);
        return null;
    }
    // Calculate wait period in milliseconds
    const waitPeriodMs = waitPeriod * 60 * 60 * 1000;
    // Calculate base next trigger time
    const baseNextTriggerTime = lastCollectionTime + waitPeriodMs;
    // Add a random delay (between RANDOM_DELAY_MIN and RANDOM_DELAY_MAX minutes)
    const randomDelayMs = getRandomDelay();
    const nextTriggerTime = baseNextTriggerTime + randomDelayMs;
    const now = Date.now();
    // Log a more concise message
    // console.log(
    //   `Next trigger: ${new Date(
    //     nextTriggerTime
    //   ).toLocaleString()} (based on last collection: ${new Date(
    //     lastCollectionTime
    //   ).toLocaleString()}, wait period: ${waitPeriod}h, random delay: ${Math.round(
    //     randomDelayMs / 60000
    //   )}m)`
    // );
    // If the calculated time is in the past, return current time + random delay
    if (nextTriggerTime < now) {
        const adjustedTime = now + randomDelayMs;
        // console.log(
        //   `⚠️ Calculated time is in the past, adjusting to ${new Date(
        //     adjustedTime
        //   ).toLocaleString()} (now + ${Math.round(
        //     randomDelayMs / 60000
        //   )}m random delay)`
        // );
        return adjustedTime;
    }
    return nextTriggerTime;
}
/**
 * Fetch account status from storage
 * @param accountId The account ID
 * @returns Promise that resolves to the account status
 */
async function fetchAccountStatus(accountId) {
    return await chrome.storage.local.get([
        STORAGE_KEYS.COLLECTION_STATUS(accountId),
        STORAGE_KEYS.FAIL_COUNT(accountId),
        STORAGE_KEYS.TIMESTAMP(accountId),
        STORAGE_KEYS.WAIT_PERIOD(accountId),
        STORAGE_KEYS.IS_ACTIVE(accountId),
        STORAGE_KEYS.PAUSED(accountId),
        STORAGE_KEYS.MANUAL_ALARM(accountId),
    ]);
}
/**
 * Check if an account should be skipped during collection
 * @param accountId The account ID
 * @param status The account status
 * @returns Promise that resolves to true if the account should be skipped
 */
async function shouldSkipAccount(accountId, status) {
    // Use destructuring for cleaner code and faster property access
    const { [STORAGE_KEYS.FAIL_COUNT(accountId)]: failCount = 0, [STORAGE_KEYS.MANUAL_ALARM(accountId)]: isManualAlarmSet = false, [STORAGE_KEYS.COLLECTION_STATUS(accountId)]: collectionStatus, [STORAGE_KEYS.IS_ACTIVE(accountId)]: isActive, [STORAGE_KEYS.PAUSED(accountId)]: isPaused, [STORAGE_KEYS.TIMESTAMP(accountId)]: timestamp = null, [STORAGE_KEYS.WAIT_PERIOD(accountId)]: waitPeriod = 24, [STORAGE_KEYS.ALARM_STATUS(accountId)]: alarmStatus, } = status;
    // Quick check for most common skip conditions first (short-circuit evaluation)
    if (!isActive || isPaused || isManualAlarmSet) {
        // Only log for manual alarms as they're special cases
        if (isManualAlarmSet) {
            console.log(`⏭️ Skipping ${accountId}: Account is manual alarm set`);
        }
        return true;
    }
    // Check collection status
    const busyStatuses = ["collecting", "queued", "waitingforresolve"];
    if (busyStatuses.includes(collectionStatus)) {
        // Only log for non-standard statuses that might need attention
        if (collectionStatus === "waitingforresolve") {
            console.log(`⏭️ Skipping ${accountId}: Collection status is ${collectionStatus}`);
        }
        return true;
    }
    // Check if a retry alarm is set for this account
    // This prevents immediate re-queueing after a failure
    if (alarmStatus === "retry") {
        // console.log(`⏭️ Skipping ${accountId}: Retry alarm is already set`);
        return true;
    }
    // Check fail count - only log high fail counts that need attention
    if (failCount > 15) {
        // console.log(`⏭️ Skipping ${accountId}: Fail count is ${failCount} (> 8)`);
        return true;
    }
    // Process timestamp if it exists - timestamp represents last successful collection
    if (timestamp) {
        const timeSinceLastCollection = Date.now() - new Date(timestamp).getTime();
        const waitPeriodMs = waitPeriod * 60 * 60 * 1000;
        // If not enough time has passed since last successful collection, skip
        if (timeSinceLastCollection < waitPeriodMs) {
            return true;
        }
    }
    // Account should not be skipped
    return false;
}
/**
 * Calculate the appropriate alarm time for an account
 * @param accountId The account ID
 * @param timestamp The last collection timestamp
 * @param waitPeriod The wait period in hours
 * @param failCount The fail count
 * @returns The calculated alarm time and whether it's a retry
 */
function calculateAlarmTime(accountId, timestamp, waitPeriod, failCount) {
    // Only consider it a retry if failCount is greater than 0
    // This ensures that if a value update was successful and reset the failCount to 0,
    // we'll set a regular alarm instead of a retry alarm
    const isRetry = failCount > 0;
    let when;
    if (isRetry) {
        // For retry alarms, use exponential backoff
        const retryDelay = getRetryDelay(failCount);
        when = Date.now() + retryDelay;
        // Keep only this log as it's useful for debugging retry issues
        if (failCount > 3) {
            console.log(`Account ${accountId} has fail count ${failCount}, using retry delay of ${Math.round(retryDelay / 60000)} minutes`);
        }
    }
    else if (timestamp) {
        // We have a previous collection time, calculate next trigger
        const nextTriggerTime = calculateNextTriggerTime(timestamp, waitPeriod);
        if (nextTriggerTime) {
            when = nextTriggerTime;
        }
        else {
            // Fallback to random delay if calculation fails
            const randomDelay = getRandomDelay();
            when = Date.now() + randomDelay;
            // Improved logging with more precise time information
            console.log(`⏰ Set random delay alarm for ${accountId} - will trigger in ${(randomDelay / 60000).toFixed(2)} minutes (${new Date(when).toLocaleString()})`);
        }
    }
    else {
        // No previous collection, use random delay
        const randomDelay = getRandomDelay();
        when = Date.now() + randomDelay;
        // Improved logging with more precise time information
        console.log(`⏰ Set random delay alarm for ${accountId} - will trigger in ${(randomDelay / 60000).toFixed(2)} minutes (${new Date(when).toLocaleString()})`);
    }
    return { when, isRetry };
}
/**
 * Set a collection alarm for an account
 * @param accountId The account ID
 * @param when The time when the alarm should fire
 * @param isRetry Whether this is a retry alarm
 * @param force Whether to force set the alarm regardless of account status
 */
async function setCollectionAlarm(accountId, when, isRetry = false, force = false) {
    try {
        const alarmName = `${ALARM_TYPES.COLLECTION}${accountId}`;
        // Clear any existing alarm first
        await chrome.alarms.clear(alarmName);
        // Always check if account is active, even if force is true
        const { [STORAGE_KEYS.IS_ACTIVE(accountId)]: isActive = true } = await chrome.storage.local.get(STORAGE_KEYS.IS_ACTIVE(accountId));
        if (isActive === false) {
            console.log(`⏭️ Not setting alarm for inactive account ${accountId}`);
            return;
        }
        // Also check if account is paused
        const { [STORAGE_KEYS.PAUSED(accountId)]: isPaused = false } = await chrome.storage.local.get(STORAGE_KEYS.PAUSED(accountId));
        if (isPaused) {
            console.log(`⏭️ Not setting alarm for paused account ${accountId}`);
            return;
        }
        // Use the provided 'when' time directly - random delay is already added in calculateNextTriggerTime
        const finalWhen = when;
        // Create the new alarm
        await chrome.alarms.create(alarmName, {
            when: finalWhen,
        });
        const alarmType = isRetry ? "retry" : "regular";
        // console.log(
        //   `⏰ ${forceText}Set ${alarmType} alarm for ${accountId} - will trigger in ${timeUntilAlarm} minutes (${new Date(
        //     finalWhen
        //   ).toLocaleString()})`
        // );
        // Update the alarm status in storage
        await chrome.storage.local.set({
            [STORAGE_KEYS.ALARM_STATUS(accountId)]: {
                type: alarmType,
                setAt: Date.now(),
                scheduledFor: finalWhen,
            },
        });
    }
    catch (error) {
        console.error(`❌ Failed to set alarm for ${accountId}:`, error);
    }
}
/**
 * Process accounts in batches to avoid overwhelming the system
 * @param accountIds Array of account IDs to process
 * @param processFn Function to process each account
 * @param batchSize Number of accounts to process in parallel
 * @returns Promise that resolves to the results of processing each account
 */
async function processBatch(accountIds, processFn, batchSize = 5) {
    const results = [];
    for (let i = 0; i < accountIds.length; i += batchSize) {
        const batch = accountIds.slice(i, i + batchSize);
        const batchResults = await Promise.all(batch.map(async (accountId) => {
            try {
                return await processFn(accountId);
            }
            catch (error) {
                console.error(`❌ Error processing ${accountId}:`, error);
                return null;
            }
        }));
        results.push(...batchResults);
        // Small delay between batches to prevent CPU spikes
        if (i + batchSize < accountIds.length) {
            await new Promise((resolve) => setTimeout(resolve, 100));
        }
    }
    return results;
}

;// CONCATENATED MODULE: ./src/extension/managers/alarm/StorageAlarmManager.ts


class StorageAlarmManager {
    constructor() { }
    /**
     * Handle the result of a collection
     * @param accountId The account ID
     */
    async handleCollectionResult(accountId) {
        try {
            // Get current status - we only read data, we don't modify it
            const status = await fetchAccountStatus(accountId);
            const currentTimestamp = status[STORAGE_KEYS.TIMESTAMP(accountId)];
            const waitPeriod = status[STORAGE_KEYS.WAIT_PERIOD(accountId)] || 24;
            // Get the most up-to-date fail count directly from storage
            // This ensures we have the latest value after any updates
            const { [STORAGE_KEYS.FAIL_COUNT(accountId)]: failCount = 0 } = await chrome.storage.local.get(STORAGE_KEYS.FAIL_COUNT(accountId));
            // Get the most up-to-date collection status
            const { [STORAGE_KEYS.COLLECTION_STATUS(accountId)]: collectionStatus = "idle", } = await chrome.storage.local.get(STORAGE_KEYS.COLLECTION_STATUS(accountId));
            // Determine success based on failCount and collection status
            // - If collection status is "success", it's a success
            // - If failCount is 0, a value update occurred, so it's a success
            const isSuccess = failCount === 0;
            console.log(`🔄 Handling collection result for ${accountId}: ${isSuccess ? "success" : "failure"} (status: ${collectionStatus}, failCount: ${failCount})`);
            if (isSuccess) {
                // For successful collections, use the current timestamp and failCount of 0
                const { when, isRetry } = calculateAlarmTime(accountId, currentTimestamp, waitPeriod, 0 // Always use 0 for successful collections
                );
                await setCollectionAlarm(accountId, when, isRetry);
                console.log(`⏰ Set regular alarm for ${accountId} at ${new Date(when).toLocaleString()}`);
            }
            else {
                // For failed collections, use the current timestamp and failCount
                console.log(`ℹ️ Using current failCount for ${accountId}: ${failCount}`);
                // Set alarm based on failCount
                const { when, isRetry } = calculateAlarmTime(accountId, currentTimestamp, waitPeriod, failCount);
                // Log the appropriate message based on whether it's a retry or not
                if (isRetry) {
                    console.log(`⏰ Set retry alarm for ${accountId} at ${new Date(when).toLocaleString()} (failCount: ${failCount})`);
                }
                else {
                    console.log(`⏰ Set regular alarm for ${accountId} at ${new Date(when).toLocaleString()} (failCount: ${failCount}, but using regular schedule)`);
                }
                await setCollectionAlarm(accountId, when, isRetry);
            }
        }
        catch (error) {
            console.error(`❌ Error handling collection result for ${accountId}:`, error);
        }
    }
    /**
     * Check and set alarms for all accounts
     * This ensures that all accounts have the proper alarms set
     */
    async checkAndSetAlarmsForAllAccounts() {
        try {
            console.log("🔄 Starting alarm check for all accounts");
            // Get all accounts from UserKeys_* entries
            const storage = await chrome.storage.local.get(null);
            // First, collect all accounts that have UserKeys
            const userKeyAccounts = Object.keys(storage)
                .filter((key) => key.startsWith("UserKeys_"))
                .map((key) => key.replace("UserKeys_", ""));
            // Then, collect all accounts that have manual alarms set
            const manualAlarmKeys = Object.keys(storage)
                .filter((key) => key.startsWith("manualAlarmSet_") && storage[key] === true)
                .map((key) => key.replace("manualAlarmSet_", ""));
            // Combine both lists, removing duplicates
            const accounts = [...new Set([...userKeyAccounts, ...manualAlarmKeys])];
            // console.log(
            //   `🔍 Found ${accounts.length} accounts to check (${userKeyAccounts.length} with UserKeys, ${manualAlarmKeys.length} with manual alarms)`
            // );
            if (accounts.length === 0) {
                console.log("ℹ️ No accounts found, skipping alarm check");
                return;
            }
            // Track how many alarms were fixed
            let fixedCount = 0;
            let manualAlarmCount = 0;
            // Process accounts in batches to avoid overwhelming the system
            const BATCH_SIZE = 10;
            for (let i = 0; i < accounts.length; i += BATCH_SIZE) {
                const batch = accounts.slice(i, i + BATCH_SIZE);
                // Process each account in the batch
                await Promise.all(batch.map(async (accountId) => {
                    try {
                        // Check if a manual alarm is set for this account
                        const manualAlarmKey = STORAGE_KEYS.MANUAL_ALARM(accountId);
                        const { [manualAlarmKey]: isManualAlarmSet = false } = await chrome.storage.local.get(manualAlarmKey);
                        // Skip accounts with manual alarms
                        if (isManualAlarmSet) {
                            // Check if the manual alarm exists
                            const alarmName = `${ALARM_TYPES.COLLECTION}${accountId}`;
                            const alarm = await chrome.alarms.get(alarmName);
                            if (alarm) {
                                // console.log(
                                //   `✅ Manual alarm for ${accountId} exists and is being preserved`
                                // );
                            }
                            else {
                                // console.log(
                                //   `⚠️ Manual alarm flag set for ${accountId} but alarm doesn't exist - checking alarm status`
                                // );
                                // Check if there's alarm status information
                                const alarmStatusKey = STORAGE_KEYS.ALARM_STATUS(accountId);
                                const { [alarmStatusKey]: alarmStatus } = await chrome.storage.local.get(alarmStatusKey);
                                if (alarmStatus &&
                                    alarmStatus.type === "manual" &&
                                    alarmStatus.scheduledFor) {
                                    // Re-create the manual alarm
                                    console.log(`🔄 Re-creating manual alarm for ${accountId}`);
                                    await chrome.alarms.create(`${ALARM_TYPES.COLLECTION}${accountId}`, {
                                        when: alarmStatus.scheduledFor,
                                    });
                                }
                            }
                            manualAlarmCount++;
                            return;
                        }
                        // Check if account is paused
                        const pausedKey = STORAGE_KEYS.PAUSED(accountId);
                        const { [pausedKey]: isPaused } = await chrome.storage.local.get(pausedKey);
                        // Check if account is active
                        const isActiveKey = STORAGE_KEYS.IS_ACTIVE(accountId);
                        const { [isActiveKey]: isActive } = await chrome.storage.local.get(isActiveKey);
                        // Skip paused, inactive, or accounts without UserKeys
                        if (isPaused) {
                            // console.log(
                            //   `⏭️ Skipping alarm check for ${accountId}: Account is paused`
                            // );
                            return;
                        }
                        if (isActive === false) {
                            console.log(`⏭️ Skipping alarm check for ${accountId}: Account is inactive`);
                            return;
                        }
                        // Check if account has UserKeys - only required for non-manual accounts
                        const userKeysKey = `UserKeys_${accountId}`;
                        const { [userKeysKey]: userKeys } = await chrome.storage.local.get(userKeysKey);
                        if (!userKeys) {
                            console.log(`⏭️ Skipping alarm check for ${accountId}: No UserKeys found`);
                            return;
                        }
                        // Check if alarm exists
                        const alarmName = `${ALARM_TYPES.COLLECTION}${accountId}`;
                        const alarm = await chrome.alarms.get(alarmName);
                    }
                    catch (error) {
                        console.error(`❌ Error checking alarm for ${accountId}:`, error);
                    }
                }));
            }
            console.log(`✅ Alarm check completed: Fixed ${fixedCount} out of ${accounts.length} alarms, preserved ${manualAlarmCount} manual alarms`);
        }
        catch (error) {
            console.error("❌ Error checking alarms for all accounts:", error);
        }
    }
}

;// CONCATENATED MODULE: ./src/extension/data/accountActionSequence.ts
const accountActionSequences = {
    STK: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "login",
            },
            {
                action: "type",
                slug: "STK",
                userSelector: "input[name='emailOrName']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                element: "button",
                textContents: "login",
            },
            {
                action: "click",
                element: "svg",
                attribute: "id",
                selectorText: "google",
            },
        ],
        postLogin: [
            {
                action: "click",
                selectorText: "wallet",
                attribute: "data-test",
                element: "button",
                duration: 12000,
            },
            {
                action: "click",
                selectorText: "wallet",
                textContents: "Daily Bonus",
                attribute: "data-test",
                element: "button",
            },
            {
                action: "click",
                selectorText: "contents",
                textContents: "claim",
                attribute: "class",
                element: "div",
            },
            {
                action: "navigate",
                URL: "https://www.stake.us",
            },
            {
                action: "click",
                selectorText: "contents",
                textContents: "claim",
                attribute: "class",
                element: "div",
            },
            {
                action: "click",
                selectorText: "gold",
                attribute: "title",
                element: "span",
            },
            {
                action: "click",
                attribute: "title",
                element: "span",
                selectorText: "sweeps",
                duration: 3000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "icon",
                slug: "STK",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "coin",
                duration: 8000,
            },
        ],
    },
    FC: {
        email: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "log",
                parentSelectorText: "header",
                parentElement: "div",
                parentAttribute: "class",
            },
            {
                action: "type",
                slug: "FC",
                userSelector: "input[name='username']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "login",
                attribute: "class",
                element: "button",
                textContents: "log",
            },
        ],
        google: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "log",
                parentSelectorText: "header",
                parentElement: "div",
                parentAttribute: "class",
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "FC",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                selectorText: "terms-conditions",
                attribute: "id",
                element: "input",
                duration: 13000,
            },
            {
                action: "click",
                element: "button",
                textContents: "confirm",
            },
            {
                action: "click",
                selectorText: "close",
                element: "button",
                attribute: "class",
            },
            {
                action: "click",
                selectorText: "coin-store-button",
                element: "button",
                attribute: "class",
                duration: 12000,
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "daily",
                textContents: "collect",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "daily",
            },
            {
                action: "click",
                selectorText: "close",
                element: "button",
                attribute: "class",
            },
            {
                action: "findVal",
                element: "span",
                slug: "FC",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "desktop",
                duration: 11000,
            },
        ],
    },
    Chumba: {
        email: [
            {
                action: "click",
                selectorText: "login",
                attribute: "href",
                element: "a",
                textContents: "log",
            },
            {
                action: "type",
                slug: "Chumba",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "nav",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "hud",
            },
            {
                action: "click",
                selectorText: "buy",
                attribute: "class",
                element: "button",
                textContents: "get",
            },
            {
                action: "click",
                selectorText: "tab",
                attribute: "class",
                element: "label",
                textContents: "daily",
            },
            {
                action: "click",
                selectorText: "daily",
                attribute: "id",
                element: "button",
                textContents: "claim",
            },
            {
                action: "navigate",
                URL: "https://lobby.chumbacasino.com/",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "value",
                slug: "Chumba",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "toggle",
                duration: 8000,
            },
        ],
    },
    Cha: {
        email: [
            {
                action: "click",
                selectorText: "header",
                attribute: "class",
                element: "button",
                textContents: "log",
            },
            {
                action: "type",
                slug: "Cha",
                userSelector: "input[id='email-address']",
                passSelector: "input[type='password']",
            },
            {
                action: "click",
                selectorText: "btn",
                attribute: "class",
                element: "button",
                textContents: "sign in",
                duration: 8000,
            },
        ],
        postLogin: [
            {
                action: "click",
                attribute: "class",
                element: "span",
                textContents: "wallet",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "inline",
                attribute: "class",
                element: "button",
                textContents: "daily",
            },
            {
                action: "click",
                selectorText: "btn",
                attribute: "class",
                element: "button",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "yes",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "span",
                slug: "Cha",
                parentElement: "button",
                parentAttribute: "id",
                parentSelectorText: "headlessui",
                duration: 8000,
            },
        ],
    },
    SPRT: {
        email: [
            {
                action: "click",
                selectorText: "login",
                attribute: "class",
                element: "button",
                textContents: "log",
                duration: 3000,
            },
            {
                action: "type",
                slug: "SPRT",
                userSelector: "input[id='emailAddress']",
                passSelector: "input[id='password']",
                duration: 6000,
            },
            {
                action: "click",
                selectorText: "login-form",
                attribute: "class",
                element: "button",
                textContents: "log",
                parentSelectorText: "login-button",
                parentElement: "div",
                parentAttribute: "class",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "log",
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "SPRT",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "input",
                attribute: "id",
                selectorText: "terms",
                duration: 14000,
            },
            {
                action: "click",
                textContents: "confirm",
                duration: 2000,
            },
            {
                action: "click",
                selectorText: "header",
                attribute: "class",
                element: "span",
                textContents: "free",
                duration: 12000,
            },
            {
                action: "click",
                selectorText: "coin-store",
                attribute: "class",
                element: "button",
                textContents: "collect",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "daily-bonus",
            },
            {
                action: "navigate",
                URL: "https://sportzino.com/",
            },
            {
                action: "click",
                selectorText: "balance-currency",
                attribute: "class",
                element: "span",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "balance",
                slug: "SPRT",
                parentElement: "span",
                parentAttribute: "class",
                parentSelectorText: "balance-switcher",
                duration: 8000,
            },
        ],
    },
    GP: {
        email: [
            {
                action: "navigate",
                URL: "https://play.globalpoker.com/login?platform=globalpoker.com",
            },
            {
                action: "type",
                slug: "GP",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "submit",
                attribute: "class",
                element: "span",
                textContents: "log in",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "navigate",
                URL: "https://play.globalpoker.com/login?platform=globalpoker.com",
            },
            {
                action: "click",
                element: "div",
                textContents: "google",
            },
        ],
        postLogin: [
            {
                action: "click",
                selectorText: "cashier-button",
                attribute: "id",
                element: "button",
                textContents: "coins",
            },
            {
                action: "click",
                selectorText: "daily-bonus",
                attribute: "id",
                element: "div",
            },
            {
                action: "navigate",
                URL: "https://play.globalpoker.com/",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "b",
                slug: "GP",
                parentElement: "div",
                parentAttribute: "id",
                parentSelectorText: "sc-balance",
                duration: 10000,
            },
        ],
    },
    Zula: {
        email: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "log",
            },
            {
                action: "type",
                slug: "Zula",
                userSelector: "input[name='username']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "log in",
                duration: 9000,
            },
        ],
        google: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "log",
            },
            {
                action: "click",
                element: "button",
                textContents: "Google",
                accountId: "Zula",
                duration: 5000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "input",
                attribute: "id",
                selectorText: "terms",
                duration: 18000,
            },
            {
                action: "click",
                textContents: "confirm",
                duration: 3000,
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "buttonBuy",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "headerButtons",
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "collect",
                textContents: "collect",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "coinsContainer",
            },
            {
                action: "click",
                selectorText: "close",
                element: "button",
                attribute: "class",
            },
            {
                action: "click",
                selectorText: "SC",
                element: "img",
                attribute: "alt",
            },
            {
                action: "findVal",
                element: "span",
                slug: "Zula",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "Decimals",
                duration: 10000,
            },
        ],
    },
    Modo: {
        email: [
            {
                action: "click",
                attribute: "class",
                element: "button",
                textContents: "Log",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "class",
                element: "span",
                textContents: "xyzxyz",
                duration: 14000,
            },
            {
                action: "type",
                slug: "Modo",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 7000,
            },
            {
                action: "click",
                attribute: "class",
                element: "span",
                textContents: "log in",
                duration: 5000,
            },
        ],
        google: [
            {
                action: "click",
                element: "button",
                textContents: "login",
            },
            {
                action: "click",
                element: "div",
                textContents: "google",
                duration: 4000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "button",
                textContents: "repair",
                duration: 12000,
            },
            {
                action: "click",
                element: "span",
                textContents: "laim",
                duration: 2000,
            },
            {
                action: "findVal",
                element: "span",
                slug: "Modo",
                parentElement: "p",
                parentAttribute: "class",
                parentSelectorText: "root",
            },
        ],
    },
    RR: {
        email: [
            {
                action: "navigate",
                URL: "https://www.rollingriches.com/",
                duration: 10000,
            },
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "Login",
                duration: 8000,
            },
            {
                action: "type",
                slug: "RR",
                userSelector: "input[id='email']",
                passSelector: "input[id='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "submit",
                attribute: "type",
                element: "button",
                textContents: "sign in",
                duration: 9000,
            },
        ],
        google: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "log",
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "RR",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                textContents: "Get coins",
                element: "span",
                duration: 7000,
            },
            {
                action: "click",
                selectorText: "daily",
                element: "label",
                attribute: "id",
                textContents: "bonus",
            },
            {
                action: "click",
                selectorText: "btn",
                element: "button",
                attribute: "class",
                textContents: "claim",
            },
            {
                action: "click",
                selectorText: "close-icon",
                attribute: "class",
                element: "span",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "",
                selectorText: "",
                slug: "RR",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "sc-coin",
                duration: 8000,
            },
        ],
    },
    WowVegas: {
        email: [
            {
                action: "click",
                selectorText: "login",
                attribute: "href",
                element: "a",
                textContents: "login",
            },
            {
                action: "type",
                slug: "WowVegas",
                userSelector: "input[id='email']",
                passSelector: "input[id='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "log in",
                duration: 7000,
            },
        ],
        google: [
            {
                action: "click",
                selectorText: "login",
                attribute: "href",
                element: "a",
                textContents: "login",
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "WowVegas",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "span",
                textContents: "wow zone",
            },
            {
                action: "click",
                selectorText: "wheel",
                attribute: "class",
                element: "div",
                duration: 10000,
                usePointerEvents: false,
            },
            {
                action: "navigate",
                URL: "https://www.wowvegas.com",
                duration: 12000,
            },
            {
                action: "click",
                selectorText: "VBC",
                attribute: "alt",
                element: "img",
                duration: 6000,
            },
            {
                action: "click",
                selectorText: "WOC",
                attribute: "alt",
                element: "img",
                duration: 5000,
            },
            {
                action: "findVal",
                element: "div",
                attribute: "class",
                selectorText: "flex",
                slug: "WowVegas",
                parentElement: "button",
                parentAttribute: "id",
                parentSelectorText: "headlessui",
                duration: 10000,
            },
        ],
    },
    Sweep: {
        email: [
            {
                action: "click",
                selectorText: "login",
                attribute: "class",
                element: "div",
                textContents: "sign in",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Sweep",
                userSelector: "input[id='poplogin_email']",
                passSelector: "input[id='poplogin_password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "login",
                attribute: "id",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "claim",
                attribute: "id",
                element: "div",
                textContents: "claim",
            },
            {
                action: "click",
                selectorText: "close",
                attribute: "id",
                element: "div",
            },
            {
                action: "click",
                selectorText: "close",
                attribute: "id",
                element: "i",
            },
            {
                action: "click",
                selectorText: "btn",
                attribute: "class",
                element: "div",
                textContents: "claim",
            },
            {
                action: "click",
                selectorText: "close",
                attribute: "id",
                element: "div",
            },
            {
                action: "click",
                selectorText: "whl-btn",
                attribute: "id",
                element: "div",
            },
            {
                action: "click",
                selectorText: "wheel_spin",
                attribute: "id",
                element: "div",
            },
            {
                action: "click",
                selectorText: "wheel_spin",
                attribute: "id",
                element: "div",
            },
            {
                action: "navigate",
                URL: "https://sweepslots.com/",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "gct",
                slug: "Sweep",
                duration: 10000,
            },
        ],
    },
    Hi5: {
        email: [
            {
                action: "click",
                element: "div",
                textContents: "log in",
            },
            {
                action: "click",
                element: "div",
                textContents: "log in with",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Hi5",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "signInButtonPopup",
            },
        ],
        postLogin: [
            {
                action: "click",
                selectorText: "sweeps-claim",
                attribute: "class",
                element: "button",
                duration: 7000,
            },
            {
                action: "click",
                selectorText: "sweeps",
                attribute: "class",
                element: "div",
                textContents: "Claim",
            },
            {
                action: "click",
                selectorText: "sweeps-claim",
                attribute: "class",
                element: "button",
            },
            {
                action: "click",
                selectorText: "sweeps",
                attribute: "class",
                element: "div",
                textContents: "Claim",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "sweeps-claim",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "sweeps-balance",
                slug: "Hi5",
                duration: 8000,
            },
        ],
    },
    Pulsz: {
        email: [],
        postLogin: [
            {
                action: "click",
                selectorText: "check",
                attribute: "role",
                element: "input",
            },
            {
                action: "click",
                element: "button",
                textContents: "agree",
            },
            {
                action: "click",
                attribute: "class",
                element: "button",
                textContents: "claim",
            },
            {
                action: "navigate",
                URL: "https://www.pulsz.com",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "data-test",
                slug: "Pulsz",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "curency",
            },
        ],
    },
    VegasGems: {
        email: [
            {
                action: "navigate",
                URL: "https://vegasgems.com/rewards/daily/claim",
            },
            {
                action: "navigate",
                URL: "https://vegasgems.com",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "",
                selectorText: "",
                slug: "VegasGems",
                parentElement: "span",
                parentAttribute: "class",
                parentSelectorText: "user-balance",
                duration: 10000,
            },
            // {
            //   action: "click",
            //   selectorText: "close",
            //   element: "button",
            //   attribute: "class",
            //   duration: 600000,
            // },
        ],
    },
    Spree: {
        email: [
            {
                action: "click",
                element: "span",
                textContents: "laim",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Spree",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "submit",
                attribute: "type",
                element: "button",
                parentElement: "form",
                parentAttribute: "data-cy",
                parentSelectorText: "login",
                duration: 8000,
            },
            {
                action: "click",
                element: "input",
                attribute: "type",
                selectorText: "checkbox",
            },
            {
                action: "click",
                element: "span",
                attribute: "class",
                textContents: "accept",
            },
            {
                action: "navigate",
                URL: "https://spree.com/",
            },
            {
                action: "click",
                element: "span",
                textContents: "check",
            },
            {
                action: "click",
                element: "span",
                textContents: "accept",
                duration: 2000,
            },
            {
                action: "click",
                element: "span",
                textContents: "laim",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "number-flow-react",
                attribute: "aria-label",
                slug: "Spree",
                duration: 8000,
            },
        ],
    },
    Ruby: {
        email: [
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Ruby",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "spin",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "spin",
            },
            {
                action: "navigate",
                URL: "https://play.rubysweeps.com/",
            },
            {
                action: "findVal",
                element: "a",
                attribute: "class",
                selectorText: "balance",
                slug: "Ruby",
                duration: 10000,
            },
        ],
    },
    Jefe: {
        email: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "login",
                parentAttribute: "class",
                parentSelectorText: "header",
                parentElement: "div",
            },
            {
                action: "type",
                slug: "Jefe",
                userSelector: "input[id='email']",
                passSelector: "input[id='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "submit",
                attribute: "type",
                element: "button",
                textContents: "sign",
                parentAttribute: "class",
                parentSelectorText: "touched",
                parentElement: "form",
                duration: 9000,
            },
        ],
        google: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "login",
                parentAttribute: "class",
                parentSelectorText: "header",
                parentElement: "div",
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "Jefe",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "div",
                textContents: "accept",
            },
            {
                action: "click",
                selectorText: "getcoin",
                attribute: "class",
                element: "button",
                parentSelectorText: "header",
                parentElement: "div",
                parentAttribute: "class",
            },
            {
                action: "click",
                selectorText: "daily",
                element: "label",
                attribute: "id",
                textContents: "daily",
            },
            {
                action: "click",
                selectorText: "btn",
                element: "button",
                attribute: "class",
                textContents: "claim",
            },
            {
                action: "click",
                selectorText: "close-icon",
                attribute: "class",
                element: "span",
            },
            {
                action: "navigate",
                URL: "https://www.jefebet.com/",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "",
                selectorText: "",
                slug: "Jefe",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "sc-coin",
                duration: 11000,
            },
        ],
    },
    RealPrize: {
        email: [
            {
                action: "click",
                selectorText: "menulogin",
                attribute: "class",
                element: "div",
                textContents: "login",
                duration: 8000,
            },
            {
                action: "type",
                slug: "RealPrize",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "login",
                attribute: "id",
                element: "button",
                duration: 7000,
            },
        ],
        google: [
            {
                action: "click",
                selectorText: "menulogin",
                attribute: "class",
                element: "div",
                textContents: "login",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "RealPrize",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "btn",
                textContents: "claim",
                duration: 5000,
            },
            {
                action: "navigate",
                URL: "https://realprize.com/",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "btn",
                textContents: "claim",
                duration: 5000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "gct",
                slug: "RealPrize",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "gcnum",
                duration: 9000,
            },
        ],
    },
    Crown: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "log in",
            },
            {
                action: "type",
                slug: "Crown",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "button",
                textContents: "log in",
                duration: 10000,
            },
        ],
        google: [
            {
                action: "click",
                element: "button",
                textContents: "log in",
            },
            {
                action: "click",
                element: "button",
                textContents: "Google",
                accountId: "Crown",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "div",
                attribute: "id",
                selectorText: "close",
                duration: 11000,
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "button",
                duration: 5000,
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "menu",
                textContents: "rewards",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "button",
                textContents: "view",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "day",
                textContents: "day",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "day",
                textContents: "day",
            },
            {
                action: "click",
                element: "div",
                attribute: "id",
                selectorText: "close",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "button",
                textContents: "laim",
                duration: 3000,
            },
            {
                action: "click",
                element: "div",
                textContents: "pin",
                duration: 5000,
            },
            {
                action: "click",
                element: "div",
                textContents: "pin",
                duration: 5000,
            },
            {
                action: "click",
                element: "picture",
                attribute: "class",
                selectorText: "game-type",
            },
            {
                action: "findVal",
                element: "div",
                attribute: "id",
                selectorText: "balance",
                slug: "Crown",
                parentElement: "div",
                parentAttribute: "id",
                parentSelectorText: "balance",
                duration: 10000,
            },
        ],
    },
    LuckyBird: {
        email: [
            {
                action: "click",
                element: "div",
                textContents: "sign in",
            },
            {
                action: "type",
                slug: "LuckyBird",
                userSelector: "input[type='text']",
                passSelector: "input[type='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "sign in",
            },
            {
                action: "click",
                element: "span",
                textContents: "daily bonus",
                parentElement: "button",
                parentAttribute: "type",
                parentSelectorText: "button",
            },
            {
                action: "click",
                element: "span",
                textContents: "redeem",
                parentElement: "li",
                parentAttribute: "class",
                parentSelectorText: "redeem",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                textContents: "daily bonus",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "nav",
            },
            {
                action: "click",
                element: "span",
                textContents: "daily bonus",
                parentElement: "button",
                parentAttribute: "type",
                parentSelectorText: "button",
            },
            {
                action: "click",
                element: "picture",
                attribute: "class",
                selectorText: "game-type",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "amount",
                slug: "LuckyBird",
            },
        ],
    },
    PulszBingo: {
        email: [],
        postLogin: [
            {
                action: "click",
                selectorText: "check",
                attribute: "role",
                element: "input",
            },
            {
                action: "click",
                textContents: "agree",
                element: "button",
            },
            {
                action: "click",
                attribute: "id",
                element: "canvas",
                selectorText: "canvas",
                duration: 8000,
            },
            {
                action: "click",
                element: "canvas",
                duration: 4000,
            },
            {
                action: "click",
                element: "div",
                attribute: "data-test",
                selectorText: "wheel",
                duration: 4000,
            },
            {
                action: "navigate",
                URL: "https://www.pulszbingo.com/",
            },
            {
                action: "click",
                element: "canvas",
                duration: 4000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "data-test",
                slug: "PulszBingo",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "curency",
                duration: 12000,
            },
        ],
    },
    ClubsPoker: {
        email: [
            {
                action: "click",
                element: "div",
                attribute: "class",
                textContents: "login",
                duration: 9000,
            },
            {
                action: "type",
                slug: "ClubsPoker",
                userSelector: "input[name='username']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "sign_in",
                textContents: "log in",
                duration: 9000,
            },
        ],
        google: [
            {
                action: "click",
                element: "div",
                attribute: "class",
                textContents: "login",
                duration: 9000,
            },
            {
                action: "click",
                textContents: "Google",
                accountId: "ClubsPoker",
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "Button",
                textContents: "get coins",
                duration: 13000,
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "Button",
                textContents: "collect now",
                duration: 12000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "value",
                slug: "ClubsPoker",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "Switch",
                duration: 10000,
            },
        ],
    },
    MoonSpin: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "Log In",
            },
            {
                action: "type",
                slug: "MoonSpin",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 9000,
            },
            {
                action: "click",
                element: "button",
                attribute: "type",
                selectorText: "submit",
                textContents: "Log In",
                duration: 10000,
            },
        ],
        google: [
            {
                action: "click",
                textContents: "Google",
                accountId: "MoonSpin",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                attribute: "aria-label",
                element: "button",
                selectorText: "Bonuses",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "type",
                element: "button",
                selectorText: "button",
                textContents: "claim",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "style",
                selectorText: "pointer-events",
                slug: "MoonSpin",
                duration: 8000,
            },
        ],
    },
    Kickr: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "Log In",
            },
            {
                action: "click",
                element: "div",
                textContents: "LOG",
            },
            {
                action: "type",
                slug: "Kickr",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                element: "span",
                attribute: "class",
                selectorText: "auth",
                textContents: "log in",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                element: "div",
                textContents: "Log",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "auth",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "Kickr",
            },
        ],
        postLogin: [
            // {
            //   action: "click",
            //   element: "label",
            //   attribute: "class",
            //   selectorText: "checkbox",
            // },
            // {
            //   action: "click",
            //   element: "div",
            //   attribute: "class",
            //   selectorText: "button",
            //   textContents: "accept",
            // },
            {
                action: "click",
                element: "span",
                attribute: "class",
                selectorText: "store",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "click",
                textContents: "Daily claims",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "kickr",
                textContents: "claim",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "",
                selectorText: "",
                slug: "Kickr",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "currency",
                duration: 10000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "",
                selectorText: "",
                slug: "Kickr",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "currency",
                duration: 6000,
            },
        ],
    },
    MyPrize: {
        email: [
            {
                action: "click",
                element: "a",
                attribute: "class",
                selectorText: "login",
                textContents: "Log",
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "login",
                textContents: "Log",
            },
            {
                action: "type",
                slug: "MyPrize",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                attribute: "type",
                selectorText: "submit",
                textContents: "log",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                element: "a",
                attribute: "class",
                selectorText: "login",
                textContents: "Log",
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "MyPrize",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "bonus",
                duration: 13000,
            },
            {
                action: "click",
                textContents: "Claim",
                element: "button",
            },
            {
                action: "click",
                textContents: "spin",
                element: "button",
            },
            {
                action: "navigate",
                URL: "https://myprize.us/",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "SC",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "amount",
                slug: "MyPrize",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "balance",
                duration: 8000,
            },
        ],
    },
    Moozi: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "Log",
            },
            {
                action: "type",
                slug: "Moozi",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                element: "button",
                textContents: "Log in",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                element: "button",
                textContents: "Log",
            },
            {
                action: "click",
                element: "span",
                textContents: "Google",
                accountId: "Moozi",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                attribute: "class",
                element: "span",
                textContents: "Daily",
                duration: 12000,
            },
            {
                action: "click",
                element: "div",
                textContents: "Claim",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                slug: "Moozi",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "coin",
                duration: 8000,
            },
        ],
    },
    Punt: {
        email: [
            {
                action: "click",
                selectorText: "header",
                attribute: "class",
                element: "button",
                textContents: "log",
            },
            {
                action: "type",
                slug: "Punt",
                userSelector: "input[type='text']",
                passSelector: "input[type='password']",
            },
            {
                action: "click",
                selectorText: "btn",
                attribute: "class",
                element: "button",
                textContents: "sign",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "-",
                attribute: "class",
                element: "span",
                textContents: "wallet",
                duration: 10000,
            },
            {
                action: "click",
                selectorText: "inline",
                attribute: "class",
                element: "button",
                textContents: "daily",
            },
            {
                action: "click",
                selectorText: "btn",
                attribute: "class",
                element: "button",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "yes",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "span",
                slug: "Punt",
                parentElement: "button",
                parentAttribute: "id",
                parentSelectorText: "headlessui",
                duration: 9000,
            },
            // {
            //   action: "click",
            //   selectorText: "daily",
            //   attribute: "id",
            //   element: "button",
            //   textContents: "claim",
            //   duration: 90000,
            // },
        ],
    },
    Yay: {
        email: [
            {
                action: "click",
                selectorText: "header-lobby",
                attribute: "class",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Yay",
                userSelector: "input[name='username']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "login",
                attribute: "class",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
        ],
        google: [],
        postLogin: [
            {
                action: "click",
                selectorText: "check",
                attribute: "type",
                element: "input",
            },
            {
                action: "click",
                textContents: "confirm",
            },
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "coin store",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "collect",
                attribute: "class",
                element: "button",
                textContents: "collect",
                duration: 8000,
            },
            {
                action: "navigate",
                URL: "https://www.yaycasino.us/lobby",
            },
            {
                action: "findVal",
                element: "span",
                slug: "Yay",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "text",
                duration: 10000,
            },
        ],
    },
    YayCOM: {
        email: [
            {
                action: "click",
                selectorText: "header-lobby",
                attribute: "class",
                element: "button",
                textContents: "log",
            },
            {
                action: "type",
                slug: "YayCOM",
                userSelector: "input[name='username']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                selectorText: "login",
                attribute: "class",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
        ],
        google: [],
        postLogin: [
            {
                action: "click",
                selectorText: "check",
                attribute: "type",
                element: "input",
            },
            {
                action: "click",
                textContents: "confirm",
            },
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "coin store",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "collect",
                attribute: "class",
                element: "button",
                textContents: "collect",
            },
            {
                action: "navigate",
                URL: "https://www.yaycasino.com/lobby",
            },
            {
                action: "findVal",
                element: "span",
                slug: "YayCOM",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "text",
                duration: 10000,
            },
        ],
    },
    Scrooge: {
        email: [
            {
                action: "click",
                selectorText: "",
                attribute: "class",
                element: "button",
                textContents: "play",
            },
            {
                action: "type",
                slug: "Scrooge",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                selectorText: "submit",
                attribute: "type",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "nav",
                attribute: "class",
                element: "a",
                textContents: "daily wheel",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                textContents: "regular",
            },
            {
                action: "click",
                attribute: "alt",
                element: "img",
                selectorText: "spin",
            },
            {
                action: "navigate",
                URL: "https://scrooge.casino/",
            },
            {
                action: "findVal",
                element: "p",
                slug: "Scrooge",
                parentElement: "label",
                parentAttribute: "class",
                parentSelectorText: "form-check",
                duration: 8000,
            },
        ],
        postLogin: [],
    },
    Zoot: {
        email: [],
        google: [
            {
                action: "click",
                element: "button",
                attribute: "data-testid",
                selectorText: "google",
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "button",
                textContents: "spin",
                duration: 10000,
            },
            {
                action: "click",
                element: "div",
                textContents: "spin",
            },
            {
                action: "click",
                selectorText: "chakra",
                attribute: "class",
                element: "span",
                textContents: "daily",
                duration: 6000,
            },
            {
                action: "click",
                selectorText: "chakra",
                attribute: "class",
                element: "span",
                textContents: "Claim",
            },
            {
                action: "click",
                selectorText: "sweeps-inactive",
                attribute: "class",
                element: "div",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "hz3a",
                slug: "Zoot",
                parentElement: "div",
                parentAttribute: "id",
                parentSelectorText: "zoot-balance",
                duration: 8000,
            },
        ],
    },
    Smiles: {
        email: [
            {
                action: "click",
                element: "p",
                textContents: "login",
            },
            {
                action: "type",
                slug: "Smiles",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                element: "p",
                textContents: "log in",
                parentElement: "button",
                parentAttribute: "type",
                parentSelectorText: "submit",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                element: "p",
                textContents: "log",
            },
            {
                action: "click",
                element: "p",
                textContents: "google",
                accountId: "Smiles",
                duration: 8000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "input",
                attribute: "type",
                selectorText: "checkbox",
                duration: 11000,
            },
            {
                action: "click",
                element: "button",
                textContents: "accept",
                duration: 8000,
            },
            {
                action: "click",
                element: "p",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "click",
                element: "img",
                attribute: "class",
                selectorText: "smile-coin",
            },
            {
                action: "findVal",
                element: "div",
                attribute: "class",
                selectorText: "balance-panel",
                slug: "Smiles",
                duration: 10000,
            },
        ],
    },
    Juked: {
        email: [
            // {
            //   action: "click",
            //   selectorText: "header",
            //   attribute: "class",
            //   element: "button",
            //   textContents: "log",
            // },
            // {
            //   action: "type",
            //   slug: "Juked",
            //   userSelector: "input[type='text']",
            //   passSelector: "input[type='password']",
            // },
            // {
            //   action: "click",
            //   selectorText: "btn",
            //   attribute: "class",
            //   element: "button",
            //   textContents: "sign",
            //   duration: 8000,
            // },
            {
                action: "click",
                selectorText: "-",
                attribute: "class",
                element: "span",
                textContents: "wallet",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "inline",
                attribute: "class",
                element: "button",
                textContents: "Hourly",
            },
            {
                action: "click",
                selectorText: "btn",
                attribute: "class",
                element: "button",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "button",
                attribute: "type",
                element: "button",
                textContents: "yes",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "span",
                slug: "Juked",
                parentElement: "button",
                parentAttribute: "id",
                parentSelectorText: "headlessui",
                duration: 8000,
            },
        ],
    },
    Spinpals: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "login",
                duration: 5000,
            },
            {
                action: "click",
                element: "button",
                textContents: "log in",
                duration: 2000,
            },
            {
                action: "type",
                slug: "Spinpals",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 7000,
            },
            {
                action: "click",
                element: "button",
                textContents: "Log",
                parentElement: "div",
                parentAttribute: "data-headlessui-state",
                parentSelectorText: "open",
                duration: 8000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "input",
                attribute: "id",
                selectorText: "check",
            },
            {
                action: "click",
                element: "button",
                textContents: "accept",
            },
            {
                action: "navigate",
                URL: "https://www.spinpals.com/user/store",
            },
            {
                action: "click",
                element: "a",
                textContents: "daily",
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "bold",
                textContents: "claim",
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "bold",
                textContents: "continue",
            },
            // {
            //   action: "click",
            //   textContents: "claim",
            // },
            {
                action: "click",
                element: "button",
                textContents: "xyzxyz",
                duration: 16000,
            },
            {
                action: "click",
                element: "button",
                textContents: "ontinue",
                duration: 2000,
            },
            {
                action: "navigate",
                URL: "https://www.spinpals.com/spin-wheel",
            },
            {
                action: "click",
                element: "canvas",
                usePointerEvents: true,
            },
            {
                action: "findVal",
                element: "p",
                attribute: "class",
                selectorText: "xs",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "urple",
                slug: "Spinpals",
            },
        ],
    },
    GoldSlips: {
        email: [
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "log",
            },
            {
                action: "type",
                slug: "GoldSlips",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
            },
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "spin",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "block",
                attribute: "class",
                element: "span",
                textContents: "spin",
            },
            {
                action: "navigate",
                URL: "https://play.goldslips.com/",
            },
            {
                action: "findVal",
                element: "a",
                attribute: "class",
                selectorText: "balance",
                slug: "GoldSlips",
                duration: 10000,
            },
        ],
    },
    iCasino: {
        email: [
            {
                action: "click",
                element: "a",
                textContents: "log in",
            },
            {
                action: "type",
                slug: "iCasino",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 7000,
            },
            {
                action: "click",
                element: "div",
                textContents: "log",
                duration: 7000,
            },
            {
                action: "navigate",
                URL: "https://app.icasino.com/",
            },
            {
                action: "click",
                element: "svg",
                attribute: "class",
                selectorText: "menu",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "rewards",
            },
            {
                action: "click",
                element: "span",
                textContents: "view",
            },
            {
                action: "click",
                element: "span",
                attribute: "class",
                selectorText: "day",
            },
            {
                action: "navigate",
                URL: "https://app.icasino.com/",
            },
            {
                action: "click",
                element: "div",
                attribute: "id",
                selectorText: "coin-switch",
            },
            {
                action: "findVal",
                element: "div",
                attribute: "data-testid",
                selectorText: "balance",
                slug: "iCasino",
                duration: 10000,
            },
        ],
        google: [
            {
                action: "click",
                element: "a",
                textContents: "log in",
            },
            {
                action: "click",
                element: "div",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "iCasino",
            },
            {
                action: "click",
                element: "div",
                textContents: "continue",
            },
            {
                action: "click",
                element: "span",
                textContents: "continue",
            },
            {
                action: "navigate",
                URL: "https://app.icasino.com/",
            },
            {
                action: "click",
                element: "svg",
                attribute: "class",
                selectorText: "menu",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "rewards",
            },
            {
                action: "click",
                element: "span",
                textContents: "view",
            },
            {
                action: "click",
                element: "span",
                attribute: "class",
                selectorText: "day",
            },
            {
                action: "navigate",
                URL: "https://app.icasino.com/",
            },
            {
                action: "click",
                element: "div",
                attribute: "id",
                selectorText: "coin-switch",
            },
            {
                action: "findVal",
                element: "div",
                attribute: "data-testid",
                selectorText: "balance",
                slug: "iCasino",
                duration: 10000,
            },
        ],
    },
    Legendz: {
        email: [
            // {
            //   action: "click",
            //   attribute: "class",
            //   element: "button",
            //   textContents: "log",
            // },
            // {
            //   action: "type",
            //   slug: "Legendz",
            //   userSelector: "input[id='login-email']",
            //   passSelector: "input[id='login-password']",
            // },
            // {
            //   action: "click",
            //   element: "button",
            //   textContents: "log",
            //   duration: 8000,
            // },
            {
                action: "click",
                element: "svg",
                attribute: "xmlns",
                selectorText: "2000",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "main",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                textContents: "Play",
            },
            {
                action: "click",
                selectorText: "spin",
                attribute: "style",
                element: "circle",
                duration: 1000,
            },
            //9x below
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            // {
            //   action: "click",
            //   selectorText: "spin",
            //   attribute: "style",
            //   element: "circle",
            //   duration: 1000,
            // },
            //
            {
                action: "navigate",
                URL: "https://www.legendz.com/",
            },
            {
                action: "findVal",
                element: "div",
                attribute: "class",
                selectorText: "css",
                parentElement: "header",
                parentAttribute: "data-testid",
                parentSelectorText: "main",
                slug: "Legendz",
                duration: 10000,
            },
        ],
    },
    SpinSorcery: {
        email: [
            {
                action: "click",
                element: "a",
                textContents: "log",
                duration: 10000,
            },
            {
                action: "type",
                slug: "SpinSorcery",
                userSelector: "input[id='email']",
                passSelector: "input[id='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "submit",
                attribute: "type",
                element: "button",
                textContents: "log",
                duration: 10000,
            },
            {
                action: "click",
                element: "button",
                textContents: "laim",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                textContents: "ok",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "laim",
                duration: 3000,
            },
            {
                action: "navigate",
                URL: "https://game.spinsorcery.net",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "p",
                attribute: "class",
                selectorText: "text-xs",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "text-xs",
                slug: "SpinSorcery",
                duration: 10000,
            },
        ],
    },
    Vivaro: {
        email: [
            {
                action: "click",
                selectorText: "text",
                attribute: "data-testid",
                element: "div",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Vivaro",
                userSelector: "input[id='username']",
                passSelector: "input[id='password']",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "text",
                attribute: "data-testid",
                element: "div",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "click",
                selectorText: "name",
                attribute: "class",
                element: "div",
                textContents: "bonus",
                duration: 6000,
            },
            {
                action: "click",
                element: "span",
                textContents: "auto",
                duration: 7000,
            },
            {
                action: "click",
                element: "span",
                textContents: "auto",
                duration: 5000,
            },
            {
                action: "navigate",
                URL: "https://www.vivaro.us/en-us/frame/daily-bonus",
            },
            // {
            //   action: "click",
            //   element: "span",
            //   textContents: "auto",
            //   duration: 7000,
            // },
            {
                action: "click",
                selectorText: "currency",
                attribute: "data-testid",
                element: "div",
                duration: 10000,
            },
            {
                action: "findVal",
                element: "div",
                attribute: "class",
                selectorText: "name",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "CurrencySwitcher",
                slug: "Vivaro",
                duration: 10000,
            },
        ],
    },
    McLuck: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 12000,
            },
            {
                action: "type",
                slug: "McLuck",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
                duration: 7000,
            },
        ],
        google: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                element: "span",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "McLuck",
            },
            {
                action: "click",
                element: "span",
                textContents: "UserKeys",
                accountId: "McLuck",
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
                duration: 7000,
            },
        ],
        postLogin: [
            {
                action: "click",
                textContents: "close",
                duration: 8000,
            },
            {
                action: "navigate",
                URL: "https://www.mcluck.com/gold-coins",
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "daily",
                textContents: "claim",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "daily",
                textContents: "rewards",
                duration: 6000,
            },
            {
                action: "click",
                element: "p",
                textContents: "day",
                duration: 6000,
            },
            {
                action: "click",
                element: "button",
                textContents: "xyz",
                duration: 18000,
            },
            {
                action: "navigate",
                URL: "https://www.mcluck.com/home",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "data-test",
                selectorText: "sc",
                slug: "McLuck",
                duration: 7000,
            },
        ],
    },
    HelloMillions: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "type",
                slug: "HelloMillions",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
            },
        ],
        google: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                element: "span",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "HelloMillions",
            },
            {
                action: "click",
                element: "span",
                textContents: "UserKeys",
                accountId: "HelloMillions",
            },
        ],
        postLogin: [
            // {
            //   action: "click",
            //   textContents: "close",
            //   duration: 8000,
            // },
            {
                action: "navigate",
                URL: "https://www.hellomillions.com/gold-coins",
            },
            // {
            //   action: "click",
            //   element: "span",
            //   textContents: "claim",
            //   parentElement: "button",
            //   parentAttribute: "class",
            //   parentSelectorText: "Daily",
            //   duration: 8000,
            // },
            {
                action: "click",
                textContents: "claim now",
                parentElement: "button",
                parentAttribute: "data-test",
                parentSelectorText: "open-daily-reward-button",
                duration: 7000,
            },
            {
                action: "click",
                element: "button",
                textContents: "spin & win",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                attribute: "data-test",
                selectorText: "daily",
                textContents: "claim my rewards",
                duration: 13000,
            },
            {
                action: "navigate",
                URL: "https://www.hellomillions.com/gold-coins",
                duration: 10000,
            },
            {
                action: "findVal",
                element: "span",
                slug: "HelloMillions",
                parentElement: "div",
                parentAttribute: "data-test",
                parentSelectorText: "sc-balance",
            },
        ],
    },
    SpinBlitz: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "xyzxyz",
                duration: 11000,
            },
            {
                action: "type",
                slug: "SpinBlitz",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
                duration: 7000,
            },
        ],
        google: [
            {
                action: "click",
                element: "span",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "SportsMillions",
            },
        ],
        postLogin: [
            {
                action: "navigate",
                URL: "https://www.spinblitz.com/gold-coins",
            },
            {
                action: "click",
                textContents: "close",
            },
            {
                action: "click",
                element: "span",
                textContents: "laim",
                duration: 7000,
            },
            {
                action: "click",
                textContents: "claim my",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "content",
                duration: 7000,
            },
            {
                action: "click",
                textContents: "ewards",
                duration: 13000,
            },
            {
                action: "click",
                textContents: "ewards",
                duration: 13000,
            },
            {
                action: "navigate",
                URL: "https://www.spinblitz.com/",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "data-test",
                selectorText: "sc-balance",
                slug: "SpinBlitz",
                duration: 5000,
            },
        ],
    },
    DingDingDing: {
        email: [
            {
                action: "navigate",
                URL: "https://dingdingding.com/login/",
            },
            {
                action: "click",
                element: "button",
                textContents: "xyzxyz",
                duration: 13000,
            },
            {
                action: "type",
                slug: "DingDingDing",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
                duration: 7000,
            },
            {
                action: "click",
                element: "strong",
                textContents: "log",
                duration: 7000,
            },
        ],
        google: [],
        postLogin: [
            {
                action: "click",
                attribute: "class",
                element: "span",
                selectorText: "btn",
                textContents: "daily",
                duration: 18000,
            },
            {
                action: "click",
                element: "button",
                textContents: "collect",
                duration: 7000,
            },
            {
                action: "navigate",
                URL: "https://dingdingding.com/lobby/",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "id",
                selectorText: "switch",
                parentElement: "span",
                parentAttribute: "class",
                parentSelectorText: "counter",
                slug: "DingDingDing",
            },
        ],
    },
    MegaBonanza: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "xyzxyz",
                duration: 15000,
            },
            {
                action: "type",
                slug: "MegaBonanza",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
                duration: 7000,
            },
        ],
        google: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                element: "span",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "MegaBonanza",
            },
            {
                action: "click",
                element: "div",
                textContents: "continue",
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "button",
                textContents: "close",
                duration: 5000,
            },
            {
                action: "navigate",
                URL: "https://www.megabonanza.com/gold-coins",
            },
            {
                action: "click",
                element: "span",
                textContents: "claim",
                duration: 5000,
            },
            {
                action: "click",
                element: "button",
                textContents: "rewards",
                duration: 6000,
            },
            {
                action: "click",
                element: "button",
                textContents: "xyzxyz",
                duration: 18000,
            },
            {
                action: "navigate",
                URL: "https://www.megabonanza.com/gold-coins",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "currencyValue",
                slug: "MegaBonanza",
                duration: 5000,
            },
        ],
    },
    BetCoin: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 12000,
            },
            {
                action: "type",
                slug: "MegaBonanza",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
                duration: 7000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "daily",
                textContents: "claim",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "daily",
                textContents: "rewards",
                duration: 6000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "data-test",
                selectorText: "sc-value",
                slug: "McLuck",
                duration: 18000,
            },
            {
                action: "navigate",
                URL: "https://www.mcluck.com",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "data-test",
                selectorText: "sc-value",
                slug: "MegaBonanza",
                duration: 5000,
            },
        ],
    },
    SidePot: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "Log In",
                duration: 8000,
            },
            {
                action: "type",
                slug: "SidePot",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                attribute: "type",
                selectorText: "submit",
                textContents: "Log In",
                duration: 10000,
            },
            {
                action: "click",
                attribute: "aria-label",
                element: "button",
                selectorText: "Bonuses",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "type",
                element: "button",
                selectorText: "button",
                textContents: "laim",
            },
            {
                action: "findVal",
                element: "span",
                attribute: "style",
                selectorText: "pointer-events",
                slug: "SidePot",
                duration: 8000,
            },
        ],
    },
    VegasCoins: {
        email: [
            {
                action: "navigate",
                URL: "https://www.vegascoins.com/lobby",
            },
            {
                action: "click",
                element: "span",
                textContents: "Log",
                duration: 8000,
            },
            {
                action: "type",
                slug: "VegasCoins",
                userSelector: "input[id='email']",
                passSelector: "input[id='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "Login",
                parentElement: "button",
                parentSelectorText: "submit",
                parentAttribute: "type",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                element: "span",
                textContents: "Log",
                duration: 8000,
            },
            {
                action: "click",
                element: "img",
                attribute: "src",
                selectorText: "Social2.2fd93bf5",
                accountId: "VegasCoins",
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "span",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "text",
                parentElement: "div",
                parentSelectorText: "items",
                parentAttribute: "class",
                slug: "VegasCoins",
                duration: 8000,
            },
        ],
    },
    Dara: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "Log In",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Dara",
                userSelector: "input[type='id']",
                passSelector: "input[type='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                attribute: "class",
                selectorText: "login",
                duration: 10000,
            },
        ],
        google: [
            {
                action: "click",
                element: "button",
                textContents: "Log In",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "google",
                duration: 8000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "div",
                textContents: "day",
                duration: 12000,
            },
            {
                action: "click",
                attribute: "class",
                element: "button",
                selectorText: "btn",
                textContents: "free",
                duration: 8000,
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "btn",
                textContents: "collect",
            },
            {
                action: "click",
                element: "svg",
                attribute: "data-icon",
                selectorText: "x",
            },
            {
                action: "click",
                element: "img",
                attribute: "alt",
                selectorText: "SC",
            },
            {
                action: "findVal",
                element: "div",
                attribute: "class",
                selectorText: "coin",
                slug: "Dara",
                duration: 8000,
            },
        ],
    },
    CasinoClick: {
        email: [
            {
                action: "click",
                element: "a",
                textContents: "Sign In",
                duration: 8000,
            },
            {
                action: "type",
                slug: "CasinoClick",
                userSelector: "input[name='clientIdentifier']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                attribute: "type",
                selectorText: "submit",
                textContents: "sign in",
                duration: 10000,
            },
            {
                action: "click",
                element: "a",
                textContents: "get coins",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-testid",
                element: "button",
                selectorText: "Bonus",
            },
            {
                action: "findVal",
                element: "p",
                attribute: "data-cp",
                selectorText: "Balance",
                slug: "CasinoClick",
                duration: 8000,
            },
            // {
            //   action: "click",
            //   selectorText: "daily",
            //   attribute: "id",
            //   element: "button",
            //   textContents: "claim",
            //   duration: 45000,
            // },
            //ggXM@$Q8qbaDXhp
        ],
    },
    PlayFame: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
            },
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "xyzxyz",
                duration: 12000,
            },
            {
                action: "type",
                slug: "PlayFame",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
            },
        ],
        google: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
            },
            {
                action: "click",
                element: "span",
                textContents: "google",
            },
        ],
        postLogin: [
            {
                action: "click",
                attribute: "role",
                element: "input",
                selectorText: "check",
                duration: 5000,
            },
            {
                action: "click",
                element: "button",
                textContents: "agree",
                duration: 2000,
            },
            {
                action: "navigate",
                URL: "https://www.PlayFame.com/gold-coins",
            },
            {
                action: "click",
                element: "span",
                textContents: "claim",
                duration: 5000,
            },
            {
                action: "click",
                element: "button",
                textContents: "rewards",
                duration: 6000,
            },
            {
                action: "click",
                element: "button",
                textContents: "xyz",
                duration: 16000,
            },
            {
                action: "navigate",
                URL: "https://www.PlayFame.com/home",
            },
            {
                action: "findVal",
                element: "span",
                parentElement: "div",
                parentSelectorText: "currency",
                parentAttribute: "class",
                slug: "PlayFame",
                duration: 6000,
            },
        ],
    },
    Jackpota: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "xyzxyz",
                duration: 13000,
            },
            {
                action: "type",
                slug: "Jackpota",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
                duration: 7000,
            },
        ],
        google: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
            },
            {
                action: "click",
                element: "span",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "Jackpota",
            },
            {
                action: "click",
                element: "span",
                textContents: "UserKeys",
                accountId: "Jackpota",
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "button",
                textContents: "close",
            },
            {
                action: "navigate",
                URL: "https://www.jackpota.com/gold-coins",
            },
            {
                action: "click",
                element: "span",
                textContents: "claim",
                duration: 5000,
            },
            {
                action: "click",
                element: "button",
                textContents: "claim my rewards",
                duration: 6000,
            },
            {
                action: "click",
                element: "button",
                textContents: "xyz",
                duration: 16000,
            },
            {
                action: "navigate",
                URL: "https://www.jackpota.com/home",
            },
            {
                action: "findVal",
                element: "span",
                parentElement: "div",
                parentSelectorText: "currency",
                parentAttribute: "class",
                slug: "Jackpota",
            },
        ],
    },
    SportsMillions: {
        email: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                textContents: "Log",
                duration: 5000,
            },
            {
                action: "click",
                element: "xyz",
                selectorText: "xyzxyz",
                duration: 11000,
            },
            {
                action: "type",
                slug: "SportsMillions",
                userSelector: "input[name='email']",
                passSelector: "input[name='password']",
                duration: 8000,
            },
            {
                action: "click",
                attribute: "data-test",
                element: "button",
                selectorText: "login-now",
                textContents: "log",
                duration: 7000,
            },
        ],
        google: [
            {
                action: "click",
                attribute: "data-test",
                element: "a",
                selectorText: "header-login",
                duration: 5000,
            },
            {
                action: "click",
                element: "span",
                textContents: "google",
            },
            {
                action: "click",
                element: "div",
                textContents: "UserKeys",
                accountId: "SportsMillions",
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "button",
                textContents: "close",
                duration: 2000,
            },
            {
                action: "navigate",
                URL: "https://www.SportsMillions.com/gold-coins",
            },
            {
                action: "click",
                element: "span",
                textContents: "laim",
                duration: 7000,
            },
            {
                action: "click",
                element: "button",
                textContents: "eward",
                duration: 7000,
            },
            {
                action: "click",
                element: "button",
                textContents: "eward",
                duration: 17000,
            },
            {
                action: "navigate",
                URL: "https://www.SportsMillions.com/",
            },
            {
                action: "findVal",
                element: "span",
                parentElement: "div",
                parentSelectorText: "SC",
                parentAttribute: "data-title",
                slug: "SportsMillions",
                duration: 5000,
            },
        ],
    },
    Baba: {
        email: [
            {
                action: "click",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Baba",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
                duration: 8000,
            },
            {
                action: "click",
                element: "div",
                textContents: "login",
                duration: 10000,
            },
        ],
        google: [
            {
                action: "click",
                element: "button",
                textContents: "log",
                duration: 8000,
            },
            {
                action: "click",
                element: "button",
                textContents: "google",
            },
        ],
        postLogin: [
            {
                action: "click",
                textContents: "accept",
                duration: 12000,
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "close",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "close",
            },
            {
                action: "click",
                textContents: "collect",
                duration: 8000,
            },
            {
                action: "navigate",
                URL: "https://play.babacasino.com/#/",
            },
            {
                action: "click",
                textContents: "collect",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "span",
                attribute: "class",
                selectorText: "star-inserted",
                parentElement: "div",
                parentAttribute: "class",
                parentSelectorText: "toggler",
                slug: "Baba",
            },
        ],
    },
    Funrize: {
        email: [
            {
                action: "click",
                element: "span",
                textContents: "log in",
                duration: 8000,
            },
            {
                action: "type",
                slug: "Funrize",
                userSelector: "input[data-tid='login-email-input']",
                passSelector: "input[data-tid='login-password-input']",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "continue",
                duration: 10000,
            },
            {
                action: "click",
                element: "span",
                textContents: "promotions",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "collect",
                duration: 8000,
            },
            {
                action: "navigate",
                URL: "https://www.funrize.com/promotions",
            },
            {
                action: "click",
                element: "span",
                textContents: "spin",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "spin",
                duration: 10000,
            },
            {
                action: "navigate",
                URL: "https://www.funrize.com/",
            },
            {
                action: "findVal",
                element: "span",
                parentElement: "span",
                parentSelectorText: "entries",
                parentAttribute: "data-tid",
                slug: "Funrize",
                duration: 8000,
            },
        ],
        postLogin: [],
    },
    NolimitCoins: {
        email: [
            {
                action: "click",
                element: "span",
                textContents: "log in",
                duration: 8000,
            },
            {
                action: "type",
                slug: "NolimitCoins",
                userSelector: "input[data-tid='login-email-input']",
                passSelector: "input[data-tid='login-password-input']",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "log in",
                duration: 10000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "div",
                textContents: "promo",
                parentElement: "div",
                parentSelectorText: "class",
                parentAttribute: "reward",
            },
            {
                action: "click",
                element: "span",
                textContents: "spin",
            },
            {
                action: "click",
                element: "span",
                textContents: "spin",
            },
            {
                action: "click",
                element: "div",
                textContents: "promo",
                parentElement: "div",
                parentSelectorText: "class",
                parentAttribute: "reward",
            },
            {
                action: "click",
                element: "span",
                textContents: "claim",
            },
            {
                action: "click",
                element: "span",
                textContents: "ollect",
                duration: 8000,
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "prize",
                duration: 8000,
            },
            {
                action: "findVal",
                element: "div",
                attribute: "data-tid",
                selectorText: "entries",
                slug: "NolimitCoins",
                duration: 8000,
            },
        ],
    },
    FunzCity: {
        email: [
            {
                action: "click",
                element: "span",
                textContents: "log in",
                duration: 8000,
            },
            {
                action: "type",
                slug: "FunzCity",
                userSelector: "input[data-tid='login-email-input']",
                passSelector: "input[data-tid='login-password-input']",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "continue",
                duration: 10000,
            },
            {
                action: "click",
                element: "span",
                textContents: "promotions",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "claim",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "collect",
                duration: 8000,
            },
            {
                action: "navigate",
                URL: "https://www.funzcity.com/promotions",
            },
            {
                action: "click",
                element: "span",
                textContents: "spin",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "spin",
                duration: 10000,
            },
            {
                action: "navigate",
                URL: "https://www.funzcity.com/",
            },
            {
                action: "findVal",
                element: "span",
                parentElement: "span",
                parentSelectorText: "entries",
                parentAttribute: "data-tid",
                slug: "FunzCity",
                duration: 8000,
            },
        ],
    },
    JackpotRabbit: {
        email: [
            {
                action: "click",
                element: "span",
                textContents: "sign in",
                duration: 8000,
            },
            {
                action: "type",
                slug: "JackpotRabbit",
                userSelector: "input[type='email']",
                passSelector: "input[type='password']",
            },
            {
                action: "click",
                element: "span",
                textContents: "sign in",
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "ring",
            },
            {
                action: "click",
                element: "p",
                textContents: "daily",
                duration: 11000,
            },
            {
                action: "click",
                element: "span",
                textContents: "reward",
            },
            {
                action: "click",
                element: "img",
                attribute: "src",
                selectorText: "prize",
                parentElement: "header",
                parentSelectorText: "logged-in",
                parentAttribute: "class",
            },
            {
                action: "click",
                element: "img",
                attribute: "src",
                selectorText: "super",
                parentElement: "header",
                parentSelectorText: "logged-in",
                parentAttribute: "class",
            },
            {
                action: "findVal",
                element: "span",
                parentElement: "div",
                parentSelectorText: "currency",
                parentAttribute: "class",
                slug: "JackpotRabbit",
                duration: 8000,
            },
        ],
    },
    LuckyHands: {
        email: [
            {
                action: "click",
                textContents: "login",
                duration: 8000,
            },
            {
                action: "type",
                slug: "LuckyHands",
                userSelector: "input[name='username']",
                passSelector: "input[name='password']",
            },
            {
                action: "click",
                element: "div",
                textContents: "login",
                duration: 8000,
            },
        ],
        google: [
            {
                action: "click",
                textContents: "login",
                duration: 8000,
            },
            {
                action: "click",
                textContents: "google",
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "ring",
                duration: 14000,
            },
            {
                action: "click",
                element: "div",
                attribute: "class",
                selectorText: "daily-bonus",
            },
            {
                action: "click",
                element: "div",
                textContents: "daily bonus",
                parentElement: "div",
                parentSelectorText: "wallet-wrapper",
                parentAttribute: "class",
            },
            {
                action: "findVal",
                element: "div",
                parentElement: "div",
                parentSelectorText: "header-wallet",
                parentAttribute: "class",
                slug: "LuckyHands",
                duration: 8000,
            },
        ],
    },
    ChipnWin: {
        email: [
            {
                action: "click",
                element: "span",
                textContents: "log in",
                duration: 8000,
            },
            {
                action: "type",
                slug: "ChipnWin",
                userSelector: "input[id='input_customemail']",
                passSelector: "input[id='input_custompassword']",
                duration: 8000,
            },
            {
                action: "click",
                element: "span",
                textContents: "log in",
                duration: 10000,
            },
        ],
        postLogin: [
            {
                action: "click",
                element: "span",
                textContents: "store",
            },
            {
                action: "click",
                element: "span",
                textContents: "start",
                parentElement: "div",
                parentSelectorText: "class",
                parentAttribute: "reward",
            },
            {
                action: "click",
                element: "span",
                textContents: "claim",
            },
            {
                action: "click",
                element: "img",
                attribute: "src",
                selectorText: "coins",
                parentSelectorText: "header",
                parentAttribute: "class",
            },
            {
                action: "click",
                element: "img",
                attribute: "src",
                selectorText: "sc",
                parentElement: "div",
                parentSelectorText: "header",
                parentAttribute: "class",
            },
            {
                action: "findVal",
                element: "p",
                selectorText: "s12",
                slug: "ChipnWin",
            },
        ],
    },
};
/* harmony default export */ const accountActionSequence = ((/* unused pure expression or super */ null && (accountActionSequences)));

;// CONCATENATED MODULE: ./src/extension/managers/collection/helpers/WindowHelper.ts
/**
 * Helper functions for window management in collection process
 *
 * Features:
 * - Window position memory between sessions
 * - Automatic cleanup of orphaned windows
 * - Support for left, right, or center positioning
 * - Cascade effect for multiple windows
 * - Screen boundary safety checks
 */
const WindowHelper = {
    // Single source of truth for account collection state
    // This centralizes all our tracking into one structure
    accountCollectionState: new Map(),
    // Reverse lookup for tabs
    tabToAccount: new Map(),
    // Track window creation times
    windowCreationTimes: new Map(),
    // Track if the check interval has been set
    hasCheckInterval: false,
    // Tracker to avoid duplicate cleanup operations
    pendingCleanups: new Set(),
    // Last master cleanup timestamp
    lastMasterCleanup: 0,
    // Cache for screen dimensions
    screenDimensionsCache: null,
    lastScreenDimensionsCheck: 0,
    SCREEN_CACHE_TTL: 60000, // 1 minute
    // Constants
    MAX_TAB_LIFETIME: 5 * 60 * 1000, // 5 minutes max for a tab to exist
    MAX_WINDOW_LIFETIME: 10 * 60 * 1000, // 10 minutes max for a window to exist
    WINDOW_CONFIG_KEY: "collectionWindowConfig", // Storage key for window configuration
    LAST_WINDOW_POSITION_KEY: "lastWindowPosition", // Storage key for last window position
    MASTER_CLEANUP_INTERVAL: 30 * 60 * 1000, // 30 minutes between master cleanups (increased from 15 minutes)
    ORPHANED_CHECK_INTERVAL: 5 * 60 * 1000, // 5 minutes between orphaned window checks (new constant)
    // Default window configuration
    DEFAULT_WINDOW_CONFIG: {
        width: 1200,
        height: 800,
        left: 100,
        top: 100,
        focused: true,
        type: "normal",
        position: "right",
        cascadeOffset: 40,
    },
    // Type definition for window configuration
    windowConfigType: {},
    // Default screen dimensions (fallback if we can't detect)
    DEFAULT_SCREEN: {
        width: 1920,
        height: 1080,
    },
    // Track window count for offset calculation
    windowCount: 0,
    maxWindowsBeforeReset: 10,
    /**
     * Check if WindowHelper is already initialized
     */
    isInitialized() {
        return (this.accountCollectionState.size > 0 ||
            this.tabToAccount.size > 0 ||
            this.windowCreationTimes.size > 0 ||
            this.hasCheckInterval);
    },
    /**
     * Initialize window monitoring
     */
    init() {
        // Prevent multiple initializations
        if (this.hasCheckInterval) {
            console.log("ℹ️ WindowHelper already has an interval set up, skipping initialization");
            return;
        }
        // Set up periodic checks for orphaned windows - using a single interval for both
        // orphaned window check and master cleanup to reduce extension load
        const intervalId = setInterval(() => {
            // Prioritize error handling and wrap in try/catch to prevent interval from being killed
            try {
                // Check orphaned windows on every interval
                this.checkForOrphanedWindows().catch((error) => {
                    console.error("❌ Error checking for orphaned windows:", error);
                });
                // Run master cleanup periodically
                const now = Date.now();
                if (now - this.lastMasterCleanup > this.MASTER_CLEANUP_INTERVAL) {
                    this.masterCleanup().catch((error) => {
                        console.error("❌ Error during master cleanup:", error);
                    });
                    this.lastMasterCleanup = now;
                }
            }
            catch (error) {
                console.error("❌ Unhandled error in WindowHelper interval:", error);
            }
        }, 60000); // Check every minute
        // Store interval ID in global extension context to prevent garbage collection
        if (typeof globalThis !== "undefined") {
            globalThis.windowHelperIntervalId = intervalId;
        }
        // Mark that we've set up the interval
        this.hasCheckInterval = true;
        console.log("✅ WindowHelper initialized with window tracking checks");
        // Listen for tab close events to clean up our tracking
        chrome.tabs.onRemoved.addListener((tabId) => {
            if (this.tabToAccount.has(tabId)) {
                console.log(`🧹 Tab ${tabId} was closed externally, cleaning up tracking`);
                this.cleanupTabTracking(tabId);
            }
        });
        // Listen for window close events to clean up tracking
        chrome.windows.onRemoved.addListener(async (windowId) => {
            this.cleanupWindowTracking(windowId);
        });
        // Run master cleanup on initialization to clean up any resources from previous sessions
        this.masterCleanup().catch((error) => {
            console.error("❌ Error during initial master cleanup:", error);
        });
        this.lastMasterCleanup = Date.now();
    },
    /**
     * Master cleanup - performs a thorough cleanup of all browser windows and tabs,
     * reconciling with our tracking data
     */
    async masterCleanup() {
        console.log("🧹 Starting master cleanup of all windows and tabs");
        try {
            // Get all windows and tabs
            const allWindows = await chrome.windows.getAll();
            const allTabs = [];
            // Collect all tabs from all windows
            for (const window of allWindows) {
                if (window.id) {
                    try {
                        const tabs = await chrome.tabs.query({ windowId: window.id });
                        allTabs.push(...tabs);
                    }
                    catch (error) {
                        console.error(`❌ Error querying tabs for window ${window.id}:`, error);
                    }
                }
            }
            // Create sets of existing IDs for quick lookup
            const existingWindowIds = new Set(allWindows.map((w) => w.id));
            const existingTabIds = new Set(allTabs.map((t) => t.id).filter((id) => id !== undefined));
            // 1. Clean up any tabs in our tracking that no longer exist
            const nonExistentTabs = [];
            for (const tabId of this.tabToAccount.keys()) {
                if (!existingTabIds.has(tabId)) {
                    nonExistentTabs.push(tabId);
                }
            }
            for (const tabId of nonExistentTabs) {
                this.cleanupTabTracking(tabId);
                console.log(`🧹 Master cleanup: Removed tracking for non-existent tab ${tabId}`);
            }
            // 2. Clean up any windows in our tracking that no longer exist
            const nonExistentWindows = [];
            for (const windowId of this.windowCreationTimes.keys()) {
                if (!existingWindowIds.has(windowId)) {
                    nonExistentWindows.push(windowId);
                }
            }
            for (const windowId of nonExistentWindows) {
                this.cleanupWindowTracking(windowId);
                console.log(`🧹 Master cleanup: Removed tracking for non-existent window ${windowId}`);
            }
            // 3. Close any orphaned extension windows that are open but not in active use
            // (Only close windows created by our extension, identified by about:blank tabs)
            for (const window of allWindows) {
                if (window.id) {
                    try {
                        const tabs = await chrome.tabs.query({ windowId: window.id });
                        // Check if this is an extension-created window (has about:blank tabs)
                        const hasAboutBlankTabs = tabs.some((tab) => tab.url === "about:blank");
                        // Check if window is being actively tracked by an account
                        let isTracked = false;
                        for (const state of this.accountCollectionState.values()) {
                            if (state.windowId === window.id) {
                                isTracked = true;
                                break;
                            }
                        }
                        // If it's our window but not tracked, close it
                        if (hasAboutBlankTabs && !isTracked) {
                            console.log(`🧹 Master cleanup: Closing orphaned extension window ${window.id}`);
                            try {
                                await chrome.windows.remove(window.id);
                            }
                            catch (closeError) {
                                console.error(`❌ Error closing orphaned window ${window.id}:`, closeError);
                            }
                        }
                    }
                    catch (error) {
                        console.error(`❌ Error processing window ${window.id} during master cleanup:`, error);
                    }
                }
            }
            console.log(`✅ Master cleanup completed: Processed ${allWindows.length} windows and ${allTabs.length} tabs`);
        }
        catch (error) {
            console.error("❌ Error during master cleanup:", error);
        }
    },
    /**
     * Clean up tracking for a window
     */
    cleanupWindowTracking(windowId) {
        // Clean up window creation time tracking
        this.windowCreationTimes.delete(windowId);
        // Find all accounts using this window and update their state
        for (const [accountId, state] of this.accountCollectionState.entries()) {
            if (state.windowId === windowId) {
                console.log(`🧹 Removing window ${windowId} association for account ${accountId}`);
                // Update the account state to no longer reference this window
                state.windowId = null;
                // Find and remove any tabs that were in this window
                const tabsToRemove = [];
                for (const tabId of state.tabIds) {
                    tabsToRemove.push(tabId);
                }
                // Remove the tab associations
                for (const tabId of tabsToRemove) {
                    state.tabIds.delete(tabId);
                    this.tabToAccount.delete(tabId);
                }
                // If there are no tabs left, we could optionally remove the account state
                if (state.tabIds.size === 0) {
                    // this.accountCollectionState.delete(accountId);
                    // We'll keep the entry to maintain start time
                }
            }
        }
    },
    /**
     * Get screen dimensions safely (works in background context)
     */
    async getScreenDimensions() {
        const now = Date.now();
        // Return cached dimensions if available and not expired
        if (this.screenDimensionsCache &&
            now - this.lastScreenDimensionsCheck < this.SCREEN_CACHE_TTL) {
            return this.screenDimensionsCache;
        }
        try {
            // Try to get screen info using chrome API
            try {
                const displays = await chrome.system.display.getInfo();
                if (displays && displays.length > 0) {
                    // Use the primary display or the first one
                    const primaryDisplay = displays.find((d) => d.isPrimary) || displays[0];
                    const dimensions = {
                        width: primaryDisplay.workArea.width,
                        height: primaryDisplay.workArea.height,
                    };
                    // Update cache
                    this.screenDimensionsCache = dimensions;
                    this.lastScreenDimensionsCheck = now;
                    return dimensions;
                }
            }
            catch (error) {
                console.warn("⚠️ Error getting display info:", error);
            }
            return this.DEFAULT_SCREEN;
        }
        catch (error) {
            console.error("❌ Error getting screen dimensions:", error);
            return this.DEFAULT_SCREEN;
        }
    },
    /**
     * Initialize with default window position - can specify a preferred position strategy
     */
    async initializeWithDefaultPosition(position = "right", width = 1000, height = 300) {
        this.init();
        await this.setupDefaultWindowPosition(100, 100, width, height, position);
        console.log(`✅ WindowHelper initialized with ${position} window position`);
    },
    /**
     * Set up the default window position
     */
    async setupDefaultWindowPosition(topOffset = 100, rightOffset = 100, width = 1200, height = 0, position = "right") {
        try {
            const screenDimensions = await this.getScreenDimensions();
            const left = Math.max(0, screenDimensions.width - width - rightOffset);
            await this.setWindowConfig({
                width,
                height,
                left,
                top: topOffset,
                position,
            });
            this.windowCount = 0;
        }
        catch (error) {
            console.error(`❌ Error setting default window position:`, error);
        }
    },
    /**
     * Set the window configuration for collections
     */
    async setWindowConfig(config) {
        try {
            const fullConfig = { ...this.DEFAULT_WINDOW_CONFIG, ...config };
            await chrome.storage.local.set({ [this.WINDOW_CONFIG_KEY]: fullConfig });
        }
        catch (error) {
            console.error(`❌ Error setting window configuration:`, error);
        }
    },
    /**
     * Get the current window configuration
     */
    async getWindowConfig() {
        try {
            const { [this.WINDOW_CONFIG_KEY]: config } = await chrome.storage.local.get(this.WINDOW_CONFIG_KEY);
            return config || this.DEFAULT_WINDOW_CONFIG;
        }
        catch (error) {
            console.error(`❌ Error getting window configuration:`, error);
            return this.DEFAULT_WINDOW_CONFIG;
        }
    },
    /**
     * Get tab info safely
     */
    async getTabSafely(tabId) {
        try {
            return await chrome.tabs.get(tabId);
        }
        catch (error) {
            return null;
        }
    },
    /**
     * Get window safely
     */
    async getWindowSafely(windowId) {
        try {
            return await chrome.windows.get(windowId);
        }
        catch (error) {
            return null;
        }
    },
    /**
     * Check if window exists
     */
    async windowExists(windowId) {
        return (await this.getWindowSafely(windowId)) !== null;
    },
    /**
     * Check if tab exists
     */
    async tabExists(tabId) {
        return (await this.getTabSafely(tabId)) !== null;
    },
    /**
     * Get all tabs associated with an account
     */
    getTabsForAccount(accountId) {
        const state = this.accountCollectionState.get(accountId);
        if (!state)
            return [];
        return Array.from(state.tabIds);
    },
    /**
     * Remember the position and dimensions of a window
     *
     * This saves the current window position to storage so it can be
     * restored in future sessions.
     *
     * @param windowId The ID of the window to remember
     */
    async rememberWindowPosition(windowId) {
        try {
            // Get the current window state
            const windowInfo = await this.getWindowSafely(windowId);
            if (!windowInfo) {
                console.log(`⚠️ Cannot remember position of non-existent window ${windowId}`);
                return;
            }
            // Extract position and dimensions
            const { left, top, width, height } = windowInfo;
            if (left === undefined ||
                top === undefined ||
                width === undefined ||
                height === undefined) {
                console.log(`⚠️ Window ${windowId} has incomplete position information`);
                return;
            }
            // Store the position
            const lastPosition = {
                left,
                top,
                width,
                height,
                timestamp: Date.now(),
            };
            // Save to storage
            await chrome.storage.local.set({
                [this.LAST_WINDOW_POSITION_KEY]: lastPosition,
            });
            console.log(`✅ Remembered position of window ${windowId}: ${JSON.stringify(lastPosition)}`);
        }
        catch (error) {
            console.error(`❌ Error remembering window position:`, error);
        }
    },
    /**
     * Get the last remembered window position, if available
     *
     * Returns the saved window position from a previous session,
     * or null if no position was saved or it's invalid.
     */
    async getLastWindowPosition() {
        try {
            const { [this.LAST_WINDOW_POSITION_KEY]: lastPosition } = await chrome.storage.local.get(this.LAST_WINDOW_POSITION_KEY);
            if (!lastPosition) {
                console.log(`ℹ️ No previous window position found`);
                return null;
            }
            // Validate position data
            if (lastPosition.left === undefined ||
                lastPosition.top === undefined ||
                lastPosition.width === undefined ||
                lastPosition.height === undefined) {
                console.log(`⚠️ Stored window position is incomplete, ignoring`);
                return null;
            }
            console.log(`ℹ️ Found previous window position: ${JSON.stringify(lastPosition)}`);
            return lastPosition;
        }
        catch (error) {
            console.error(`❌ Error getting last window position:`, error);
            return null;
        }
    },
    /**
     * Clears any saved window position data
     *
     * Useful if you want to reset to default positioning
     */
    async clearSavedWindowPosition() {
        try {
            await chrome.storage.local.remove(this.LAST_WINDOW_POSITION_KEY);
            console.log(`✅ Cleared saved window position data`);
        }
        catch (error) {
            console.error(`❌ Error clearing window position:`, error);
        }
    },
    /**
     * Create a dedicated collection window
     */
    async createDedicatedWindow(width = 0, // Use config value if not specified
    height = 0 // Use config value if not specified
    ) {
        try {
            // Get current config to use for defaults
            const config = await this.getWindowConfig();
            // Check for last used position
            const lastPosition = await this.getLastWindowPosition();
            let useLastPosition = false;
            // Get screen dimensions to calculate position
            const screenDimensions = await this.getScreenDimensions();
            // Use provided values, last position, or fall back to config
            let finalWidth = width || config.width;
            let finalHeight = height || config.height;
            // Validate last position - check if it's within the current screen
            // Only use last position if it's recent (within 30 days)
            if (lastPosition) {
                const thirtyDaysMs = 30 * 24 * 60 * 60 * 1000;
                const isRecent = Date.now() - lastPosition.timestamp < thirtyDaysMs;
                // We'll use the last position regardless of screen boundaries
                if (isRecent) {
                    useLastPosition = true;
                }
                else {
                    console.log(`ℹ️ Last window position is too old, using default positioning`);
                    await this.clearSavedWindowPosition();
                }
            }
            if (useLastPosition && lastPosition) {
                if (!width)
                    finalWidth = lastPosition.width;
                if (!height)
                    finalHeight = lastPosition.height;
            }
            // Calculate position - either use last position or calculate based on config
            let left, top;
            if (useLastPosition && lastPosition) {
                // Use exactly the last position without any offset
                left = lastPosition.left;
                top = lastPosition.top;
            }
            else {
                // Calculate position based on configuration without any offset
                top = config.top;
                switch (config.position) {
                    case "left":
                        left = config.left;
                        break;
                    case "right":
                        left = Math.max(0, screenDimensions.width - finalWidth - config.left);
                        break;
                    case "center":
                        left = (screenDimensions.width - finalWidth) / 2;
                        break;
                    default:
                        left = config.left;
                }
            }
            // Create window
            try {
                const newWindow = await chrome.windows.create({
                    url: "about:blank",
                    focused: true,
                    type: config.type,
                    width: finalWidth,
                    height: finalHeight,
                    left,
                    top,
                    incognito: false,
                });
                if (!newWindow?.id) {
                    throw new Error("Failed to create window");
                }
                const windowId = newWindow.id;
                // Track window creation time
                this.windowCreationTimes.set(windowId, Date.now());
                // Try to position the window precisely
                try {
                    await chrome.windows.update(windowId, {
                        left,
                        top,
                        width: finalWidth,
                        height: finalHeight,
                    });
                    // Add listener to remember window position when moved or resized
                    if (!chrome.windows.onBoundsChanged) {
                        // If the onBoundsChanged event isn't available, we'll remember the position now
                        await this.rememberWindowPosition(windowId);
                    }
                    else {
                        // Add a one-time listener to capture window position when it's moved/resized
                        const boundsChangedListener = async (windowObj) => {
                            if (windowObj.id === windowId) {
                                await this.rememberWindowPosition(windowId);
                                chrome.windows.onBoundsChanged.removeListener(boundsChangedListener);
                            }
                        };
                        chrome.windows.onBoundsChanged.addListener(boundsChangedListener);
                    }
                }
                catch (error) {
                    console.warn(`⚠️ Could not position window:`, error);
                }
                return windowId;
            }
            catch (initialError) {
                // If the window creation fails with a bounds error, try creating it without position parameters
                if (initialError.message && initialError.message.includes("bounds")) {
                    console.log(`ℹ️ Window creation failed with bounds error, trying without position parameters`);
                    const newWindow = await chrome.windows.create({
                        url: "about:blank",
                        focused: true,
                        type: config.type,
                        incognito: false,
                    });
                    if (!newWindow?.id) {
                        throw new Error("Failed to create window in fallback attempt");
                    }
                    const windowId = newWindow.id;
                    // Track window creation time
                    this.windowCreationTimes.set(windowId, Date.now());
                    return windowId;
                }
                else {
                    // Re-throw any non-bounds related errors
                    throw initialError;
                }
            }
        }
        catch (error) {
            console.error(`❌ Error creating collection window:`, error);
            throw error;
        }
    },
    /**
     * Create and activate a tab for a specific account
     */
    async createAndActivateTab(accountId, url) {
        // Validate parameters
        if (!accountId) {
            throw new Error("Missing accountId parameter");
        }
        if (!url) {
            throw new Error(`Missing URL parameter for account: ${accountId}`);
        }
        try {
            // Initialize account state if it doesn't exist
            if (!this.accountCollectionState.has(accountId)) {
                this.accountCollectionState.set(accountId, {
                    windowId: null,
                    tabIds: new Set(),
                    startTime: Date.now(),
                });
            }
            // Get current account state
            const accountState = this.accountCollectionState.get(accountId);
            // Clean up any existing resources for this account
            await this.cleanupAccountResources(accountId);
            // Create a new dedicated window for isolation
            const windowId = await this.createDedicatedWindow();
            // Update account state with new window
            accountState.windowId = windowId;
            console.log(`🔄 Created window ${windowId} for account ${accountId}`);
            // Create a tab in the window
            const tab = await chrome.tabs.create({
                url,
                active: true,
                windowId,
            });
            if (!tab?.id) {
                throw new Error("Failed to create tab");
            }
            // Wait for tab to stabilize
            await new Promise((resolve) => setTimeout(resolve, 2000));
            // Activate tab and window
            try {
                await chrome.tabs.update(tab.id, { active: true });
                await chrome.windows.update(windowId, { focused: true });
            }
            catch (error) {
                console.error(`❌ Error activating tab or window:`, error);
            }
            // Initialize cache if possible
            await this.initTabCache(tab.id);
            // Track this tab
            accountState.tabIds.add(tab.id);
            this.tabToAccount.set(tab.id, accountId);
            console.log(`✅ Created tab ${tab.id} in window ${windowId} for account ${accountId}`);
            // Set a timeout for this tab
            this.setTabTimeout(tab.id);
            return tab.id;
        }
        catch (error) {
            console.error(`❌ Error creating tab for account ${accountId}:`, error);
            throw error;
        }
    },
    /**
     * Initialize tab cache
     */
    async initTabCache(tabId) {
        try {
            if (chrome.scripting) {
                await chrome.scripting.executeScript({
                    target: { tabId },
                    func: () => {
                        sessionStorage.clear();
                        window.location.reload();
                    },
                });
                await new Promise((resolve) => setTimeout(resolve, 3000));
            }
        }
        catch (error) {
            console.warn(`⚠️ Could not initialize cache for tab ${tabId}:`, error);
        }
    },
    /**
     * Set a timeout for a tab to ensure it doesn't stay open indefinitely
     */
    setTabTimeout(tabId) {
        // Find the account for this tab
        const accountId = this.tabToAccount.get(tabId);
        if (!accountId)
            return;
        // Get account state
        const accountState = this.accountCollectionState.get(accountId);
        if (!accountState)
            return;
        // Set new timeout - we don't need to track it in the account state
        // since we can clean up based on tab ID
        setTimeout(async () => {
            console.log(`⏱️ Tab timeout reached for ${accountId} (tab ${tabId})`);
            await this.closeTab(tabId);
        }, this.MAX_TAB_LIFETIME);
    },
    /**
     * Close a tab and clean up its tracking
     */
    async closeTab(tabId) {
        console.log(`🔄 Attempting to close tab ${tabId}`);
        // Find the account for this tab
        const accountId = this.tabToAccount.get(tabId);
        if (!accountId) {
            console.log(`ℹ️ Tab ${tabId} is not tracked, skipping close`);
            return;
        }
        // Remove tracking for this tab first to prevent race conditions
        this.tabToAccount.delete(tabId);
        // Update account state
        const accountState = this.accountCollectionState.get(accountId);
        if (accountState) {
            accountState.tabIds.delete(tabId);
        }
        // Now attempt to close the tab
        try {
            const tab = await this.getTabSafely(tabId);
            if (tab) {
                await chrome.tabs.remove(tabId);
                console.log(`✅ Closed tab ${tabId} for account ${accountId}`);
            }
            else {
                console.log(`ℹ️ Tab ${tabId} no longer exists, just cleaned up tracking`);
            }
        }
        catch (error) {
            console.log(`ℹ️ Error closing tab ${tabId}: ${error}`);
        }
    },
    /**
     * Forcefully close a tab by ID, ignoring tracking data
     * Used for emergency cleanup when tabs are stuck
     */
    async forceCloseTab(tabId) {
        try {
            // Don't worry about tracking here - this is a last-resort method
            await chrome.tabs.remove(tabId);
            console.log(`🔥 Force-closed tab ${tabId}`);
            // Clean up tracking if it exists
            this.cleanupTabTracking(tabId);
            return true;
        }
        catch (error) {
            console.log(`❌ Failed to force-close tab ${tabId}: ${error}`);
            return false;
        }
    },
    /**
     * Clean up tracking for a tab
     */
    cleanupTabTracking(tabId) {
        const accountId = this.tabToAccount.get(tabId);
        if (!accountId)
            return;
        // Remove from tab tracking
        this.tabToAccount.delete(tabId);
        // Update account state
        const accountState = this.accountCollectionState.get(accountId);
        if (accountState) {
            accountState.tabIds.delete(tabId);
            console.log(`🧹 Removed tab ${tabId} from tracking for account ${accountId}`);
        }
    },
    /**
     * Clean up all resources for an account
     */
    async cleanupAccountResources(accountId) {
        // Check if cleanup is already in progress for this account
        const cleanupKey = `cleanup_${accountId}`;
        if (this.pendingCleanups.has(cleanupKey)) {
            console.log(`ℹ️ Cleanup already in progress for account ${accountId}, skipping duplicate request`);
            return;
        }
        // Mark cleanup as in progress
        this.pendingCleanups.add(cleanupKey);
        try {
            console.log(`🧹 Cleaning up resources for account ${accountId}`);
            const accountState = this.accountCollectionState.get(accountId);
            if (!accountState) {
                console.log(`ℹ️ No resources found for account ${accountId}`);
                return;
            }
            // Save window position before cleanup if window exists
            if (accountState.windowId !== null) {
                await this.rememberWindowPosition(accountState.windowId);
                console.log(`💾 Saved window position for account ${accountId} before cleanup`);
            }
            // Close all tabs for this account
            const tabsToClose = [...accountState.tabIds];
            for (const tabId of tabsToClose) {
                try {
                    await this.closeTab(tabId);
                }
                catch (error) {
                    console.error(`❌ Error closing tab ${tabId} for account ${accountId}:`, error);
                    // Try force close as last resort
                    await this.forceCloseTab(tabId);
                }
            }
            // Close window if it exists and is no longer needed
            if (accountState.windowId !== null) {
                try {
                    const windowId = accountState.windowId;
                    // Check if window still exists
                    const windowExists = await this.windowExists(windowId);
                    if (windowExists) {
                        try {
                            await chrome.windows.remove(windowId);
                            console.log(`✅ Closed window ${windowId} for account ${accountId}`);
                        }
                        catch (windowError) {
                            console.error(`❌ Error closing window ${windowId}:`, windowError);
                            // Fall back to individual tab closing if window close fails
                            try {
                                const tabs = await chrome.tabs.query({ windowId });
                                for (const tab of tabs) {
                                    if (tab.id) {
                                        await this.forceCloseTab(tab.id);
                                    }
                                }
                            }
                            catch (tabQueryError) {
                                console.error(`❌ Error querying tabs for window ${windowId}:`, tabQueryError);
                            }
                        }
                    }
                    // Clean up window tracking
                    this.windowCreationTimes.delete(windowId);
                    // Update account state
                    accountState.windowId = null;
                }
                catch (error) {
                    console.error(`❌ Error closing window for ${accountId}:`, error);
                }
            }
            // Reset account state but keep the entry for tracking purposes
            accountState.tabIds.clear();
        }
        catch (error) {
            console.error(`❌ Error during account resource cleanup for ${accountId}:`, error);
        }
        finally {
            // Always remove from pending cleanups to prevent deadlocks
            this.pendingCleanups.delete(cleanupKey);
        }
    },
    /**
     * Check for windows that have been open too long or are no longer needed
     */
    async checkForOrphanedWindows() {
        try {
            // Get all existing windows
            const allWindows = await chrome.windows.getAll();
            const existingWindowIds = new Set(allWindows.map((win) => win.id));
            // Check windows in our creation time tracking
            const now = Date.now();
            for (const [windowId, creationTime,] of this.windowCreationTimes.entries()) {
                // Window no longer exists
                if (!existingWindowIds.has(windowId)) {
                    this.windowCreationTimes.delete(windowId);
                    this.cleanupWindowTracking(windowId);
                    continue;
                }
                // Window has been open too long
                if (now - creationTime > this.MAX_WINDOW_LIFETIME) {
                    try {
                        await chrome.windows.remove(windowId);
                        console.log(`✅ Closed aged window ${windowId}`);
                    }
                    catch (error) {
                        console.error(`❌ Error closing aged window ${windowId}:`, error);
                        // Try individual tab closing if window close fails
                        try {
                            const tabs = await chrome.tabs.query({ windowId });
                            for (const tab of tabs) {
                                if (tab.id) {
                                    await this.forceCloseTab(tab.id);
                                }
                            }
                        }
                        catch (tabQueryError) {
                            console.error(`❌ Error querying tabs for window ${windowId}:`, tabQueryError);
                        }
                    }
                    this.windowCreationTimes.delete(windowId);
                    this.cleanupWindowTracking(windowId);
                }
            }
        }
        catch (error) {
            console.error("❌ Error checking for orphaned windows:", error);
        }
    },
    /**
     * Clean up all resources associated with a collection
     * This method can be called from CollectionManager to centralize cleanup
     */
    async cleanupCollectionResources(accountId, currentTabId, logCallback) {
        const log = (level, message, data) => {
            if (logCallback) {
                logCallback(level, message, data);
            }
            else {
                console[level](message, data !== undefined ? data : "");
            }
        };
        log("info", `🧹 Collection cleanup: Starting cleanup for account ${accountId}`);
        try {
            // Special handling for the current tab if provided
            if (currentTabId) {
                log("info", `🧹 Collection cleanup: Handling current tab ${currentTabId}`);
                // Check if this tab is associated with this account
                const tabAccountId = this.tabToAccount.get(currentTabId);
                if (tabAccountId === accountId) {
                    try {
                        // Remember window position before closing the tab
                        const tab = await this.getTabSafely(currentTabId);
                        if (tab && tab.windowId) {
                            await this.rememberWindowPosition(tab.windowId);
                            log("info", `💾 Collection cleanup: Saved window position for tab ${currentTabId}`);
                        }
                        await this.closeTab(currentTabId);
                        log("info", `✅ Collection cleanup: Closed current tab ${currentTabId}`);
                    }
                    catch (error) {
                        log("error", `❌ Collection cleanup: Error closing current tab:`, error);
                        // Last resort force close
                        try {
                            await this.forceCloseTab(currentTabId);
                            log("info", `🔥 Collection cleanup: Force-closed current tab ${currentTabId}`);
                        }
                        catch (forceError) {
                            log("error", `❌ Collection cleanup: Failed to force-close tab:`, forceError);
                        }
                    }
                }
                else if (tabAccountId) {
                    log("warn", `⚠️ Collection cleanup: Tab ${currentTabId} belongs to account ${tabAccountId}, not ${accountId}`);
                }
                else {
                    log("info", `ℹ️ Collection cleanup: Tab ${currentTabId} is not tracked`);
                }
            }
            // Remember window position before cleaning up resources
            const accountState = this.accountCollectionState.get(accountId);
            if (accountState && accountState.windowId !== null) {
                await this.rememberWindowPosition(accountState.windowId);
                log("info", `💾 Collection cleanup: Saved window position before cleanup`);
            }
            // Clean up all remaining resources
            await this.cleanupAccountResources(accountId);
            log("info", `✅ Collection cleanup: Resources cleaned up for ${accountId}`);
        }
        catch (error) {
            log("error", `❌ Collection cleanup: Error during cleanup:`, error);
        }
    },
};
// Remove self-initialization to prevent duplicate initialization
// WindowHelper.init();

;// CONCATENATED MODULE: ./src/extension/managers/collection/helpers/NetworkHelper.ts
/**
 * Helper functions for network-related operations
 */
class NetworkHelper {
    /**
     * Check if there is an active internet connection
     * Uses multiple methods for reliability
     */
    static async checkInternetConnection() {
        try {
            // First check navigator.onLine
            if (!navigator.onLine) {
                console.log("🌐 Browser reports offline status");
                return false;
            }
            // Then try to connect to a reliable endpoint
            const controller = new AbortController();
            const timeoutId = setTimeout(() => controller.abort(), 5000); // 5 second timeout
            try {
                const response = await fetch("https://8.8.8.8", {
                    mode: "no-cors",
                    cache: "no-cache",
                    signal: controller.signal,
                });
                clearTimeout(timeoutId);
                console.log("🌐 Internet connection verified");
                return true;
            }
            catch (fetchError) {
                clearTimeout(timeoutId);
                // If the first attempt fails, try an alternative endpoint
                try {
                    const response = await fetch("https://1.1.1.1", {
                        mode: "no-cors",
                        cache: "no-cache",
                        signal: controller.signal,
                    });
                    console.log("🌐 Internet connection verified (backup endpoint)");
                    return true;
                }
                catch (backupError) {
                    console.log("🌐 Both connection checks failed");
                    return navigator.onLine; // Fall back to navigator.onLine as last resort
                }
            }
        }
        catch (error) {
            console.warn("🌐 Error checking internet connection:", error);
            return navigator.onLine; // Fall back to navigator.onLine if all else fails
        }
    }
}

;// CONCATENATED MODULE: ./src/extension/managers/collection/CollectionManager.ts




// Environment flag to control logging - set to false to reduce performance impact
const DEBUG_MODE = false; // Changed from true to false
// Cache for action sequences to avoid repeated lookups
const actionSequenceCache = new Map();
class CollectionManager {
    constructor() {
        this.currentTab = null;
        this.currentValueUpdateSuccess = false;
        // Constants for timing
        this.INITIAL_DELAY = 6000;
        this.NAVIGATION_TIMEOUT = 30000;
        this.currentTab = null;
        this.currentValueUpdateSuccess = false;
        this.initializeWindowHelper();
        chrome.runtime.onSuspend.addListener(() => this.cleanup());
    }
    /**
     * Controlled logging function to reduce console operations
     * Only logs important information unless DEBUG_MODE is enabled
     */
    log(level, message, data) {
        // Skip info logs unless DEBUG_MODE or it contains these important keywords
        const isImportantInfo = message.includes("Starting collection") ||
            message.includes("completed successfully") ||
            message.includes("Error") ||
            message.includes("❌") ||
            message.includes("Cleaning up resources");
        if (level === "info" && !DEBUG_MODE && !isImportantInfo) {
            return;
        }
        switch (level) {
            case "info":
                console.log(message, data !== undefined ? data : "");
                break;
            case "warn":
                console.warn(message, data !== undefined ? data : "");
                break;
            case "error":
                console.error(message, data !== undefined ? data : "");
                break;
        }
    }
    /**
     * Initialize WindowHelper with default window position
     */
    async initializeWindowHelper() {
        try {
            await WindowHelper.initializeWithDefaultPosition();
            this.log("info", "✅ WindowHelper initialized with default window position");
        }
        catch (error) {
            this.log("error", "❌ Error initializing WindowHelper:", error);
        }
    }
    /**
     * Set collection status in storage
     */
    async setCollectionStatus(accountId, status) {
        await chrome.storage.local.set({
            [STORAGE_KEYS.COLLECTION_STATUS(accountId)]: status,
        });
    }
    /**
     * Reset collection status to idle
     */
    async resetAllCollectionStatuses() {
        try {
            const storage = await chrome.storage.local.get(null);
            const statusKeys = Object.keys(storage).filter((key) => key.startsWith("collection_status_"));
            if (statusKeys.length === 0) {
                return;
            }
            // Batch all updates into a single storage operation
            const updates = {};
            for (const key of statusKeys) {
                updates[key] = "idle";
            }
            // Apply all updates in a single operation
            await chrome.storage.local.set(updates);
        }
        catch (error) {
            this.log("error", "❌ Error resetting collection statuses:", error);
        }
    }
    /**
     * Clean up resources for a collection
     * Increased reliability by adding retries and better error handling
     */
    async cleanup(accountId) {
        this.log("info", `🧹 Collection cleanup: ${accountId ? `for account ${accountId}` : "general cleanup"}`);
        try {
            // Delegate cleanup to WindowHelper for more centralized window/tab management
            await WindowHelper.cleanupCollectionResources(accountId || "", this.currentTab, (level, message, data) => this.log(level, message, data));
            // Reset current tab regardless of outcome
            this.currentTab = null;
            // If this is a general cleanup, run the master cleanup to ensure no resources are leaked
            if (!accountId) {
                try {
                    await WindowHelper.masterCleanup();
                }
                catch (masterCleanupError) {
                    this.log("error", "❌ Error during master cleanup:", masterCleanupError);
                }
            }
            this.log("info", `✅ Collection cleanup: Completed`);
        }
        catch (error) {
            this.log("error", `❌ Collection cleanup: Error during cleanup:`, error);
            // Retry cleanup once if it fails (only for specific account cleanups)
            if (accountId) {
                this.log("info", `🔄 Collection cleanup: Retrying cleanup for ${accountId}`);
                try {
                    await WindowHelper.cleanupAccountResources(accountId);
                }
                catch (retryError) {
                    this.log("error", `❌ Collection cleanup: Retry also failed:`, retryError);
                }
            }
            // Reset current tab on error to prevent stale references
            this.currentTab = null;
        }
    }
    /**
     * Main collection method
     */
    async performCollection(accountId, URL) {
        const startTime = Date.now();
        let tabId = null;
        this.currentValueUpdateSuccess = false;
        let actions = [];
        let collectionSuccess = false;
        this.log("info", `🚀 Collection: Starting collection for ${accountId} with URL ${URL}`);
        try {
            // Validate parameters
            if (!accountId) {
                this.log("error", `❌ Collection: Missing accountId parameter`);
                throw new Error("Missing accountId parameter");
            }
            if (!URL) {
                this.log("error", `❌ Collection: Missing URL parameter for account ${accountId}`);
                throw new Error(`Missing URL parameter for account: ${accountId}`);
            }
            // Verify internet connection - skip if it takes too long
            this.log("info", `🌐 Collection: Checking internet connection for ${accountId}`);
            const connectionPromise = NetworkHelper.checkInternetConnection();
            const timeoutPromise = new Promise((resolve) => setTimeout(() => resolve(true), 2000));
            const isConnected = await Promise.race([
                connectionPromise,
                timeoutPromise,
            ]);
            if (isConnected === false) {
                this.log("error", `❌ Collection: No internet connection for ${accountId}`);
                throw new Error("No internet connection");
            }
            // Create and activate tab
            this.log("info", `🔄 Collection: Creating tab for ${accountId}`);
            tabId = await WindowHelper.createAndActivateTab(accountId, URL);
            this.currentTab = tabId;
            this.log("info", `✅ Collection: Created tab ${tabId} for ${accountId}`);
            // Regular collection sequence
            this.log("info", `📋 Collection: Getting action sequence for ${accountId}`);
            actions = await this.getActionSequence(accountId);
            if (!actions || actions.length === 0) {
                this.log("error", `❌ Collection: No actions found for ${accountId}`);
                throw new Error(`No actions found for account: ${accountId}`);
            }
            // Execute actions
            this.log("info", `▶️ Collection: Executing actions for ${accountId}`);
            const { valueUpdateSuccess } = await this.executeActions(tabId, actions, accountId);
            this.log("info", `✅ Collection: Actions executed for ${accountId}, valueUpdateSuccess: ${valueUpdateSuccess}`);
            // Check if value was updated successfully during normal execution
            if (valueUpdateSuccess || this.currentValueUpdateSuccess) {
                this.log("info", `✅ Collection: Collection for ${accountId} completed successfully with value update`);
                collectionSuccess = true;
            }
            else {
                // Collection failed but didn't throw an error
                this.log("info", `⚠️ Collection: Collection for ${accountId} completed but no value was updated`);
            }
        }
        catch (error) {
            // Handle any errors during collection
            this.log("error", `❌ Collection: Error during collection for ${accountId}:`, error);
        }
        finally {
            // Always clean up resources
            this.log("info", `🧹 Collection: Cleaning up resources for ${accountId}`);
            await this.cleanup(accountId);
            // Check if a value update happened during cleanup
            if (this.currentValueUpdateSuccess && !collectionSuccess) {
                this.log("info", `✅ Collection: Value was updated during cleanup for ${accountId}, marking as successful`);
                collectionSuccess = true;
            }
            // Set final collection status based on success
            await this.setCollectionStatus(accountId, collectionSuccess ? "success" : "idle");
            this.log("info", `🔄 Collection: Final status set to ${collectionSuccess ? "success" : "idle"} for ${accountId}`);
            // Reset the manual alarm flag
            try {
                const manualAlarmKey = STORAGE_KEYS.MANUAL_ALARM(accountId);
                const { [manualAlarmKey]: isManualAlarmSet = false } = await chrome.storage.local.get(manualAlarmKey);
                if (isManualAlarmSet) {
                    await chrome.storage.local.set({ [manualAlarmKey]: false });
                    this.log("info", `🔄 Collection: Reset manual alarm flag for ${accountId}`);
                }
            }
            catch (error) {
                this.log("error", `❌ Error resetting manual alarm flag for ${accountId}:`, error);
            }
            this.log("info", `✅ Collection: Cleanup complete for ${accountId}`);
            // Log collection duration
            const duration = Date.now() - startTime;
            this.log("info", `⏱️ Collection: Collection for ${accountId} took ${duration}ms`);
        }
        // Return the final success status
        return collectionSuccess;
    }
    /**
     * Get action sequence for an account with caching
     */
    async getActionSequence(accountId, defaultAuthMethod = "email") {
        try {
            // Check cache first
            if (actionSequenceCache.has(accountId)) {
                return actionSequenceCache.get(accountId) || [];
            }
            // Get the stored auth method
            const authMethodKey = STORAGE_KEYS.AUTH_METHOD(accountId);
            const { [authMethodKey]: storedAuthMethod } = await chrome.storage.local.get(authMethodKey);
            // Use stored auth method if available, otherwise use default
            const authMethod = (storedAuthMethod || defaultAuthMethod);
            // Get the account's action sequences
            const accountSequences = accountActionSequences[accountId];
            if (!accountSequences) {
                throw new Error(`No action sequences found for account: ${accountId}`);
            }
            // Get the auth sequence based on method
            const authSequence = accountSequences[authMethod];
            if (!authSequence || !Array.isArray(authSequence)) {
                // Fall back to email sequence if the requested auth method sequence doesn't exist
                if (accountSequences.email && Array.isArray(accountSequences.email)) {
                    const sequence = [
                        ...accountSequences.email,
                        ...(accountSequences.postLogin || []),
                    ];
                    // Cache the sequence
                    actionSequenceCache.set(accountId, sequence);
                    return sequence;
                }
                throw new Error(`No valid action sequence found for ${accountId}`);
            }
            // Combine auth and post-login sequences
            const sequence = [...authSequence, ...(accountSequences.postLogin || [])];
            // Cache the sequence
            actionSequenceCache.set(accountId, sequence);
            return sequence;
        }
        catch (error) {
            throw error;
        }
    }
    /**
     * Handle a value update for an account
     */
    async handleValueUpdate(accountId, newValue) {
        this.log("info", `💾 Handling value update for ${accountId}:`, {
            newValue,
            timestamp: new Date().toISOString(),
        });
        try {
            // Basic validation
            if (!accountId || newValue === null || newValue === 0) {
                this.log("error", `❌ Invalid value update for ${accountId}: ${newValue}`);
                return false;
            }
            // Get current value directly from storage
            const valueKey = STORAGE_KEYS.VALUE_FOR(accountId);
            const { [valueKey]: currentValue, storedAccounts = {} } = await chrome.storage.local.get([valueKey, "storedAccounts"]);
            this.log("info", `🔄 Current value for ${accountId}: ${currentValue}, new value: ${newValue}`);
            // Exempt accounts that should always update regardless of value change
            const exemptAccounts = ["Sweep", "Crown", "Zoot"];
            const isExempt = exemptAccounts.includes(accountId);
            // Check for unchanged value - using epsilon comparison for floating point
            const isUnchanged = this.isValueUnchanged(newValue, currentValue);
            // Log the value comparison
            this.log("info", `📊 Value comparison for ${accountId}:`, {
                newValue: newValue,
                currentValue,
                isUnchanged,
            });
            // Case 1: Error case - unchanged value (unless exempt)
            if (isUnchanged && !isExempt) {
                this.log("info", `❌ Error case for ${accountId} - unchanged value`);
                // Critical: Reset this flag to ensure it's not incorrectly marked as success
                this.currentValueUpdateSuccess = false;
                return false;
            }
            // Case 2: Success case - store the new value and update related data
            this.log("info", `✅ Valid value update for ${accountId}: ${currentValue} -> ${newValue}`);
            // Mark as successful before updating collection status
            this.currentValueUpdateSuccess = true;
            // Update critical values and alarm for successful collection
            await this.updateSuccessfulCollection(accountId, newValue, storedAccounts);
            this.log("info", `✅ Value: Successfully stored value ${newValue} for account ${accountId}`);
            return true;
        }
        catch (error) {
            this.log("error", `❌ Error updating value for ${accountId}:`, error);
            // Ensure flag is reset on error
            this.currentValueUpdateSuccess = false;
            return false;
        }
    }
    /**
     * Helper function to check if a value is unchanged using epsilon comparison
     */
    isValueUnchanged(newValue, oldValue) {
        // If one value is null and the other isn't, they're different
        if (newValue === null || oldValue === null) {
            return newValue === oldValue;
        }
        // For floating point numbers, use an epsilon comparison
        // This allows for tiny differences due to floating point precision
        const epsilon = 0.0001; // Tolerance for floating point comparison
        return Math.abs(newValue - oldValue) < epsilon;
    }
    /**
     * Update storage and alarm for a successful collection
     */
    async updateSuccessfulCollection(accountId, newValue, storedAccounts) {
        // Update critical values for successful collection
        const now = new Date().toISOString();
        // Batch all updates into a single storage operation
        const updates = {
            [STORAGE_KEYS.FAIL_COUNT(accountId)]: 0,
            [STORAGE_KEYS.TIMESTAMP(accountId)]: now,
            [STORAGE_KEYS.COLLECTION_STATUS(accountId)]: "success",
        };
        // Add the new value to updates if provided
        if (newValue !== undefined) {
            updates[STORAGE_KEYS.VALUE_FOR(accountId)] = newValue;
        }
        // Apply storage updates
        await chrome.storage.local.set(updates);
        // Update stored accounts if available and a new value was provided
        if (newValue !== undefined && storedAccounts && storedAccounts[accountId]) {
            storedAccounts[accountId].value = newValue;
            storedAccounts[accountId].lastUpdated = Date.now();
            await chrome.storage.local.set({ storedAccounts });
        }
        this.log("info", `✅ Updated storage for ${accountId}: reset failCount, updated timestamp, set status to success${newValue !== undefined ? ", stored new value" : ""}`);
        // Send a message to update the alarm - catch and handle connection errors
        try {
            const updateResponse = await new Promise((resolve) => {
                chrome.runtime.sendMessage({
                    action: "UPDATE_ALARM",
                    accountId,
                    success: true,
                }, (response) => {
                    if (chrome.runtime.lastError) {
                        const errorMessage = chrome.runtime.lastError.message || "Unknown error";
                        // Don't treat connection errors as critical failures
                        if (errorMessage.includes("Could not establish connection")) {
                            this.log("info", `ℹ️ Normal connection state: ${errorMessage}. The background page may be inactive.`);
                            resolve({ success: true });
                        }
                        else {
                            this.log("warn", `⚠️ Error sending alarm update: ${errorMessage}`);
                            resolve({ error: errorMessage });
                        }
                    }
                    else {
                        resolve(response || { success: true });
                    }
                });
            });
            if (updateResponse.success) {
                this.log("info", `✅ Successfully updated alarm for ${accountId}`);
            }
            else if (updateResponse.error) {
                this.log("warn", `⚠️ Error updating alarm: ${updateResponse.error}`);
            }
        }
        catch (error) {
            this.log("warn", `⚠️ Exception sending alarm update: ${error}`);
        }
    }
    /**
     * Get the appropriate delay for an action
     */
    getActionDelay(action) {
        // If action has a specific duration, use it
        if (action.duration && typeof action.duration === "number") {
            return action.duration;
        }
        // Default delays based on action type
        const defaultDelays = {
            navigate: 10000,
            click: 11000,
            type: 10000,
            findVal: 13000,
        };
        // Use action-specific default or general default
        return defaultDelays[action.action] || 13000;
    }
    /**
     * Execute a sequence of actions
     */
    async executeActions(tabId, actions, accountId) {
        try {
            this.log("info", `▶️ Actions: Starting execution of ${actions.length} actions for ${accountId} in tab ${tabId}`);
            // Reset value update success
            this.currentValueUpdateSuccess = false;
            // Execute each action in sequence
            for (let i = 0; i < actions.length; i++) {
                const action = actions[i];
                // Skip invalid actions
                if (!action || !action.action) {
                    this.log("info", `⚠️ Actions: Skipping invalid action at index ${i} for ${accountId}`);
                    continue;
                }
                // Add accountId to action data if not present
                if (!action.accountId) {
                    action.accountId = accountId;
                }
                this.log("info", `▶️ Actions: Executing action ${i + 1}/${actions.length} (${action.action}) for ${accountId}`);
                try {
                    // Apply delay before action
                    // For first action, use INITIAL_DELAY, for others use action-specific delay
                    const delay = i === 0 ? this.INITIAL_DELAY : this.getActionDelay(action);
                    await new Promise((resolve) => setTimeout(resolve, delay));
                    // Execute the action
                    await this.executeAction(tabId, action);
                    this.log("info", `✅ Actions: Action ${action.action} executed successfully for ${accountId}`);
                }
                catch (error) {
                    this.log("error", `❌ Actions: Error executing action ${action.action} for ${accountId}:`, error);
                }
            }
            this.log("info", `🏁 Actions: All actions completed for ${accountId}, valueUpdateSuccess: ${this.currentValueUpdateSuccess}`);
            return {
                success: true,
                valueUpdateSuccess: this.currentValueUpdateSuccess,
            };
        }
        catch (error) {
            this.log("error", `❌ Actions: Error executing actions for ${accountId}:`, error);
            return {
                success: false,
                valueUpdateSuccess: false,
            };
        }
    }
    /**
     * Execute a specific action
     */
    async executeAction(tabId, action) {
        try {
            // Verify tab still exists before proceeding
            const tab = await WindowHelper.getTabSafely(tabId);
            if (!tab) {
                this.log("error", `❌ Action: Cannot execute ${action.action} - tab ${tabId} does not exist or is inaccessible`);
                throw new Error(`Tab ${tabId} does not exist or is inaccessible`);
            }
            this.log("info", `▶️ Action: Executing action ${action.action} for ${action.URL || ""}`);
            // Execute the action
            switch (action.action) {
                case "navigate":
                    if (!action.URL) {
                        throw new Error("No URL provided for navigation action");
                    }
                    await chrome.tabs.update(tabId, { url: action.URL });
                    // Wait for navigation to complete
                    await new Promise((resolve, reject) => {
                        const timeout = setTimeout(() => {
                            reject(new Error("Navigation timeout"));
                        }, this.NAVIGATION_TIMEOUT);
                        const listener = (updatedTabId, changeInfo) => {
                            if (updatedTabId === tabId && changeInfo.status === "complete") {
                                chrome.tabs.onUpdated.removeListener(listener);
                                clearTimeout(timeout);
                                resolve();
                            }
                        };
                        chrome.tabs.onUpdated.addListener(listener);
                    });
                    break;
                case "click":
                    // Check if this is a Google auth button click
                    const isGoogleAuthButton = action.textContents?.toLowerCase() === "google" && action.accountId;
                    // Relay the action to the content script
                    await this.relayActionToContentScript(tabId, action);
                    // Add additional delay after Google auth button click
                    if (isGoogleAuthButton) {
                        this.log("info", `🕒 Adding extra delay after Google auth button click for ${action.accountId}`);
                        // Wait for Google auth to initialize and process
                        await new Promise((resolve) => setTimeout(resolve, 13000));
                        this.log("info", `✅ Completed additional delay for Google auth for ${action.accountId}`);
                    }
                    break;
                case "type":
                case "findVal":
                    // Relay the action to the content script
                    await this.relayActionToContentScript(tabId, action);
                    break;
                default:
                    // Silently succeed for other unsupported action types
                    this.log("info", `ℹ️ Action: Skipping unsupported action type: ${action.action}`);
            }
        }
        catch (error) {
            this.log("error", `❌ Action: Error executing action ${action.action}:`, error);
            throw error;
        }
    }
    /**
     * Relay an action to the content script
     */
    async relayActionToContentScript(tabId, action) {
        return new Promise((resolve, reject) => {
            // First, check if the tab still exists before proceeding
            WindowHelper.getTabSafely(tabId).then((tab) => {
                if (!tab) {
                    this.log("error", `❌ Action: Cannot execute ${action.action} - tab ${tabId} does not exist or is inaccessible`);
                    reject(new Error(`Tab ${tabId} doesn't exist or is inaccessible`));
                    return;
                }
                // Tab exists, proceed with action
                const timeout = setTimeout(() => {
                    this.log("error", `⏱️ Action: Timeout executing ${action.action}`);
                    reject(new Error(`Timeout executing ${action.action}`));
                }, this.NAVIGATION_TIMEOUT);
                // Ensure we're sending all action data to the content script
                const actionData = {
                    ...action,
                    timestamp: Date.now(),
                    // Make sure duration is passed if it exists
                    duration: action.duration || undefined,
                };
                // Only log essential information in non-debug mode
                if (DEBUG_MODE) {
                    this.log("info", `📤 Action: Sending ${action.action} to content script with data:`, {
                        action: actionData.action,
                        duration: actionData.duration || "default",
                        accountId: actionData.accountId || "unknown",
                        element: actionData.element || "N/A",
                        selectorText: actionData.selectorText || "N/A",
                        URL: actionData.URL || "N/A",
                    });
                }
                chrome.tabs.sendMessage(tabId, { action: "executeAction", data: actionData }, (response) => {
                    clearTimeout(timeout);
                    if (chrome.runtime.lastError) {
                        this.log("error", `❌ Action: Error sending ${action.action} to tab ${tabId}:`, chrome.runtime.lastError);
                        reject(new Error(chrome.runtime.lastError.message));
                        return;
                    }
                    if (!response || response.error) {
                        this.log("error", `❌ Action: ${action.action} failed:`, response?.error || "No response");
                        reject(new Error(response?.error || "Action failed"));
                        return;
                    }
                    this.log("info", `✅ Action: ${action.action} executed successfully`);
                    resolve();
                });
            }, (error) => {
                // Error checking tab existence
                this.log("error", `❌ Action: Error checking tab ${tabId} existence:`, error);
                reject(new Error(`Error checking tab existence: ${error.message}`));
            });
        });
    }
}

;// CONCATENATED MODULE: ./src/extension/managers/collection/index.ts


;// CONCATENATED MODULE: ./src/extension/managers/queue/BackgroundQueueManager.ts

// Environment flag to control logging
const BackgroundQueueManager_DEBUG_MODE = false;
class BackgroundQueueManager {
    constructor(registry) {
        this.registry = registry;
        this.activeCollections = new Set();
        this.processingInterval = null;
        this.healthCheckInterval = null;
        this.isProcessing = false;
        // Cache for collection statuses to reduce storage reads
        this.statusCache = new Map();
        // Last time the queue was fetched
        this.lastQueueFetch = 0;
        // Cached queue items
        this.queueCache = [];
        // Configuration
        this.MAX_CONCURRENT = 1;
        this.QUEUE_KEY = "queue_items";
        this.PROCESS_INTERVAL = 4000; // 3 seconds
        this.HEALTH_CHECK_INTERVAL = 60000; // 1 minute
        this.QUEUE_CACHE_TTL = 2000; // 2 seconds
        this.initialize();
    }
    /**
     * Controlled logging function to reduce console operations
     */
    log(level, message, data) {
        if (!BackgroundQueueManager_DEBUG_MODE && level === "info")
            return;
        switch (level) {
            case "info":
                console.log(message, data !== undefined ? data : "");
                break;
            case "warn":
                console.warn(message, data !== undefined ? data : "");
                break;
            case "error":
                console.error(message, data !== undefined ? data : "");
                break;
        }
    }
    /**
     * Verify that the queue is empty
     * This is a safety check to ensure the queue is truly empty after initialization
     */
    async verifyQueueEmpty() {
        try {
            const { [this.QUEUE_KEY]: queue = [] } = await chrome.storage.local.get(this.QUEUE_KEY);
            if (queue.length > 0) {
                this.log("warn", `⚠️ Queue verification: Queue still contains ${queue.length} items after reset, forcing removal`, queue);
                await chrome.storage.local.remove(this.QUEUE_KEY);
                this.queueCache = [];
            }
            else {
                this.log("info", `✅ Queue verification: Queue is empty as expected`);
            }
            // Also check for any non-idle collection statuses
            const storage = await chrome.storage.local.get(null);
            const nonIdleStatuses = {};
            const statusPrefix = STORAGE_KEYS.COLLECTION_STATUS("").replace("_", "");
            Object.keys(storage).forEach((key) => {
                if ((key.startsWith("collection_status_") ||
                    key.startsWith(statusPrefix)) &&
                    storage[key] !== "idle") {
                    nonIdleStatuses[key] = storage[key];
                    // Update cache
                    const accountId = key
                        .replace(statusPrefix, "")
                        .replace("collection_status_", "");
                    this.statusCache.set(accountId, "idle");
                }
            });
            if (Object.keys(nonIdleStatuses).length > 0) {
                this.log("warn", `⚠️ Queue verification: Found ${Object.keys(nonIdleStatuses).length} non-idle collection statuses, resetting to idle`, nonIdleStatuses);
                // Reset all to idle in a single operation
                const updates = {};
                Object.keys(nonIdleStatuses).forEach((key) => {
                    updates[key] = "idle";
                });
                await chrome.storage.local.set(updates);
            }
            else {
                this.log("info", `✅ Queue verification: All collection statuses are idle as expected`);
            }
        }
        catch (error) {
            this.log("error", `❌ Error verifying queue is empty:`, error);
        }
    }
    // Core initialization
    async initialize() {
        try {
            this.log("info", `🚀 Queue manager: Starting initialization`);
            // Reset collections and states
            this.activeCollections.clear();
            this.statusCache.clear();
            this.log("info", `🧹 Queue manager: Cleared active collections`);
            // Perform a full queue reset instead of just resetting stuck states
            await this.resetQueue();
            this.log("info", `✅ Queue manager: Full queue reset completed during initialization`);
            // Start processing
            this.startProcessing();
            // Verify queue is empty as a final safety check
            await this.verifyQueueEmpty();
            this.log("info", `✅ Queue manager: Initialization complete`);
        }
        catch (error) {
            this.log("error", "❌ Queue manager initialization error:", error);
        }
    }
    // Reset any stuck collection states
    async resetStuckStates() {
        try {
            this.log("info", `🧹 Queue manager: Resetting stuck states`);
            const storage = await chrome.storage.local.get(null);
            // Find all collection locks and reset them
            const collectionLocks = Object.keys(storage).filter((key) => key.startsWith(STORAGE_KEYS.COLLECTION_LOCK("").replace("_", "")));
            if (collectionLocks.length > 0) {
                this.log("info", `🧹 Queue manager: Removing ${collectionLocks.length} collection locks`);
                await chrome.storage.local.remove(collectionLocks);
            }
            // Reset queue processing lock if it exists
            if (storage.queue_processing_lock) {
                await chrome.storage.local.remove("queue_processing_lock");
                this.log("info", `🧹 Queue manager: Removed queue processing lock`);
            }
            // Reset all collection statuses to idle
            const statusUpdates = {};
            const statusPrefix = STORAGE_KEYS.COLLECTION_STATUS("").replace("_", "");
            // Find all collection status keys - check both formats to be thorough
            const statusKeys = Object.keys(storage).filter((key) => key.startsWith(statusPrefix) || key.startsWith("collection_status_"));
            this.log("info", `🔍 Queue manager: Found ${statusKeys.length} collection status keys`);
            // Batch all status updates into a single storage operation
            for (const key of statusKeys) {
                const status = storage[key];
                if (status !== "idle") {
                    statusUpdates[key] = "idle";
                    // Update cache
                    const accountId = key
                        .replace(statusPrefix, "")
                        .replace("collection_status_", "");
                    this.statusCache.set(accountId, "idle");
                }
            }
            if (Object.keys(statusUpdates).length > 0) {
                await chrome.storage.local.set(statusUpdates);
                this.log("info", `🧹 Queue manager: Reset ${Object.keys(statusUpdates).length} collection statuses to idle`);
            }
            else {
                this.log("info", `ℹ️ Queue manager: No non-idle collection statuses found`);
            }
            // Clear the queue
            const { [this.QUEUE_KEY]: queue = [] } = storage;
            if (queue.length > 0) {
                this.log("info", `🧹 Queue manager: Clearing queue with ${queue.length} items`);
                await chrome.storage.local.remove(this.QUEUE_KEY);
                this.queueCache = [];
                this.lastQueueFetch = 0;
            }
            else {
                this.log("info", `ℹ️ Queue manager: Queue is already empty`);
            }
        }
        catch (error) {
            this.log("error", "❌ Error resetting stuck states:", error);
        }
    }
    /**
     * Reset the queue and all collection states
     * This is used when the extension is refreshed
     */
    async resetQueue() {
        try {
            this.log("info", "🔄 Queue manager: Resetting queue and collection states");
            // Clear active collections and cache
            this.activeCollections.clear();
            this.statusCache.clear();
            this.queueCache = [];
            this.lastQueueFetch = 0;
            // Reset stuck states (locks, etc.)
            await this.resetStuckStates();
            // Get all storage to find queue-related keys
            const storage = await chrome.storage.local.get(null);
            // Find all queue-related keys
            const queueRelatedKeys = Object.keys(storage).filter((key) => key === this.QUEUE_KEY ||
                key === "collection_queue" ||
                key === "current_collection" ||
                key === "queue_processing_lock" ||
                key === "queue_processing_timestamp" ||
                key.startsWith("queue_"));
            if (queueRelatedKeys.length > 0) {
                this.log("info", `🧹 Queue manager: Removing ${queueRelatedKeys.length} queue-related keys`);
                await chrome.storage.local.remove(queueRelatedKeys);
            }
            // Explicitly reset all collection statuses using CollectionManager if available
            if (this.registry.collectionManager) {
                await this.registry.collectionManager.resetAllCollectionStatuses();
                this.log("info", "✅ Queue manager: All collection statuses reset to idle");
            }
            // Verify queue is empty
            const { [this.QUEUE_KEY]: remainingQueue = [] } = await chrome.storage.local.get(this.QUEUE_KEY);
            if (remainingQueue.length > 0) {
                this.log("warn", `⚠️ Queue manager: Queue still contains ${remainingQueue.length} items after reset, forcing removal`);
                await chrome.storage.local.remove(this.QUEUE_KEY);
            }
            this.log("info", "✅ Queue manager: Queue reset complete");
        }
        catch (error) {
            this.log("error", "❌ Error resetting queue:", error);
        }
    }
    // Start processing intervals
    startProcessing() {
        // Clear existing intervals
        if (this.processingInterval)
            clearInterval(this.processingInterval);
        if (this.healthCheckInterval)
            clearInterval(this.healthCheckInterval);
        // Set up intervals
        this.processingInterval = setInterval(() => {
            this.processQueue().catch((e) => this.log("error", "❌ Queue processing error:", e));
        }, this.PROCESS_INTERVAL);
        this.healthCheckInterval = setInterval(() => {
            this.performHealthCheck().catch((e) => this.log("error", "❌ Health check error:", e));
        }, this.HEALTH_CHECK_INTERVAL);
        // Start immediately
        this.processQueue().catch((e) => this.log("error", "❌ Initial queue processing error:", e));
    }
    // Set collection status with cache
    async setStatus(accountId, status) {
        // Update cache first
        this.statusCache.set(accountId, status);
        // Then update storage
        await chrome.storage.local.set({
            [STORAGE_KEYS.COLLECTION_STATUS(accountId)]: status,
        });
    }
    // Get collection status with cache
    async getStatus(accountId) {
        // Check cache first
        if (this.statusCache.has(accountId)) {
            return this.statusCache.get(accountId) || "idle";
        }
        // If not in cache, get from storage
        const { [STORAGE_KEYS.COLLECTION_STATUS(accountId)]: status = "idle" } = await chrome.storage.local.get(STORAGE_KEYS.COLLECTION_STATUS(accountId));
        // Update cache
        this.statusCache.set(accountId, status);
        return status;
    }
    /**
     * Reset a specific collection status to idle
     * This can be used to fix a stuck collection
     */
    async resetCollectionStatus(accountId) {
        try {
            this.log("info", `🔄 Queue manager: Manually resetting status for ${accountId} to idle`);
            // Remove from active collections if present
            if (this.activeCollections.has(accountId)) {
                this.activeCollections.delete(accountId);
                this.log("info", `🧹 Queue manager: Removed ${accountId} from active collections`);
            }
            // Reset status to idle
            await this.setStatus(accountId, "idle");
            // Remove any locks for this collection
            await chrome.storage.local.remove(STORAGE_KEYS.COLLECTION_LOCK(accountId));
            this.log("info", `✅ Queue manager: Successfully reset status for ${accountId}`);
        }
        catch (error) {
            this.log("error", `❌ Error resetting collection status for ${accountId}:`, error);
        }
    }
    // PUBLIC METHODS
    /**
     * Add an account to the collection queue
     */
    async addToQueue(accountId, URL) {
        try {
            this.log("info", `🔄 Queue: Adding ${accountId} to queue with URL ${URL}`);
            if (!URL || !accountId) {
                this.log("error", `❌ Queue: Invalid parameters for ${accountId}`);
                return false;
            }
            // Get current queue with cache
            const queue = await this.getQueue();
            // Check if already in queue or collecting
            if (queue.some((item) => item.accountId === accountId)) {
                this.log("info", `⚠️ Queue: ${accountId} is already in queue, skipping`);
                return false;
            }
            const status = await this.getStatus(accountId);
            if (status === "collecting") {
                this.log("info", `⚠️ Queue: ${accountId} is already being collected (status: ${status}), skipping`);
                return false;
            }
            // Add to queue
            const updatedQueue = [
                ...queue,
                { accountId, URL, timestamp: Date.now() },
            ];
            // Update cache
            this.queueCache = updatedQueue;
            this.lastQueueFetch = Date.now();
            // Update storage
            await chrome.storage.local.set({ [this.QUEUE_KEY]: updatedQueue });
            await this.setStatus(accountId, "queued");
            this.log("info", `✅ Queue: Added ${accountId} to queue (${updatedQueue.length} total items)`);
            return true;
        }
        catch (error) {
            this.log("error", `❌ Error adding ${accountId} to queue:`, error);
            return false;
        }
    }
    /**
     * Remove an account from the queue
     */
    async removeFromQueue(accountId) {
        try {
            // Get current queue with cache
            const queue = await this.getQueue();
            // Filter out the account
            const updatedQueue = queue.filter((item) => item.accountId !== accountId);
            // Update cache
            this.queueCache = updatedQueue;
            this.lastQueueFetch = Date.now();
            // Update storage
            await chrome.storage.local.set({ [this.QUEUE_KEY]: updatedQueue });
            // Reset status if queued
            const status = await this.getStatus(accountId);
            if (status === "queued") {
                await this.setStatus(accountId, "idle");
            }
            return true;
        }
        catch (error) {
            this.log("error", `❌ Error removing ${accountId} from queue:`, error);
            return false;
        }
    }
    /**
     * Clean up resources
     */
    async dispose() {
        try {
            if (this.processingInterval) {
                clearInterval(this.processingInterval);
                this.processingInterval = null;
            }
            if (this.healthCheckInterval) {
                clearInterval(this.healthCheckInterval);
                this.healthCheckInterval = null;
            }
            this.activeCollections.clear();
            this.statusCache.clear();
            this.queueCache = [];
            this.lastQueueFetch = 0;
            await this.resetStuckStates();
        }
        catch (error) {
            this.log("error", "❌ Error disposing queue manager:", error);
        }
    }
    // PRIVATE PROCESSING METHODS
    /**
     * Get the current queue with caching
     */
    async getQueue() {
        const now = Date.now();
        // Return cached queue if it's fresh enough
        if (this.lastQueueFetch > 0 &&
            now - this.lastQueueFetch < this.QUEUE_CACHE_TTL) {
            return this.queueCache;
        }
        // Otherwise fetch from storage
        const { [this.QUEUE_KEY]: queue = [] } = await chrome.storage.local.get(this.QUEUE_KEY);
        // Update cache
        this.queueCache = queue;
        this.lastQueueFetch = now;
        return queue;
    }
    /**
     * Process the next items in queue
     */
    async processQueue() {
        // Prevent concurrent processing
        if (this.isProcessing)
            return;
        this.isProcessing = true;
        try {
            // Get queue with cache
            const queue = await this.getQueue();
            if (queue.length > 0) {
                this.log("info", `🔍 Queue processing: ${queue.length} items in queue`);
            }
            if (queue.length === 0) {
                this.isProcessing = false;
                return;
            }
            // Calculate available slots
            const availableSlots = this.MAX_CONCURRENT - this.activeCollections.size;
            if (availableSlots <= 0) {
                this.log("info", `⏳ Queue processing: No available slots (${this.activeCollections.size}/${this.MAX_CONCURRENT} active collections)`);
                this.isProcessing = false;
                return;
            }
            // Get items to process
            const itemsToProcess = queue.slice(0, availableSlots);
            this.log("info", `🔄 Queue processing: Processing ${itemsToProcess.length} items with ${availableSlots} available slots`);
            // Batch status checks to reduce storage operations
            const accountIds = itemsToProcess.map((item) => item.accountId);
            const statusKeys = accountIds.map((id) => STORAGE_KEYS.COLLECTION_STATUS(id));
            const statusData = await chrome.storage.local.get(statusKeys);
            // Update status cache
            accountIds.forEach((id) => {
                const key = STORAGE_KEYS.COLLECTION_STATUS(id);
                const status = statusData[key] || "idle";
                this.statusCache.set(id, status);
            });
            // Process items concurrently
            const processPromises = itemsToProcess.map(async (item) => {
                // Skip if already collecting
                const status = this.statusCache.get(item.accountId) || "idle";
                if (status === "collecting" ||
                    this.activeCollections.has(item.accountId)) {
                    this.log("info", `⚠️ Queue processing: Skipping ${item.accountId} as it's already being collected (status: ${status})`);
                    return;
                }
                // Add to active collections
                this.activeCollections.add(item.accountId);
                await this.setStatus(item.accountId, "collecting");
                this.log("info", `🚀 Queue processing: Starting collection for ${item.accountId} (${this.activeCollections.size}/${this.MAX_CONCURRENT} active)`);
                try {
                    // Start collection
                    if (this.registry.collectionManager) {
                        const startTime = Date.now();
                        // Call performCollection and get the actual success status
                        const success = await this.registry.collectionManager.performCollection(item.accountId, item.URL);
                        const duration = Date.now() - startTime;
                        this.log("info", `✅ Queue processing: Collection for ${item.accountId} completed with ${success ? "success" : "failure"} after ${duration}ms`);
                        // The CollectionManager now properly handles value updates and returns the correct status
                        await this.completeCollection(item.accountId);
                        await this.removeFromQueue(item.accountId);
                    }
                    else {
                        this.log("error", `❌ Queue processing: Collection manager not available for ${item.accountId}`);
                        await this.completeCollection(item.accountId);
                        await this.removeFromQueue(item.accountId);
                    }
                }
                catch (error) {
                    this.log("error", `❌ Error processing item ${item.accountId}:`, error);
                    await this.completeCollection(item.accountId);
                    await this.removeFromQueue(item.accountId);
                }
            });
            // Wait for all processing to complete
            await Promise.all(processPromises);
        }
        catch (error) {
            this.log("error", "❌ Error processing queue:", error);
        }
        this.isProcessing = false;
    }
    /**
     * Complete a collection and update its status
     */
    async completeCollection(accountId) {
        try {
            // Get all needed data in a single storage operation
            const storageKeys = [
                STORAGE_KEYS.COLLECTION_STATUS(accountId),
                STORAGE_KEYS.FAIL_COUNT(accountId),
                STORAGE_KEYS.TIMESTAMP(accountId),
                STORAGE_KEYS.WAIT_PERIOD(accountId),
            ];
            const storageData = await chrome.storage.local.get(storageKeys);
            const currentStatus = storageData[STORAGE_KEYS.COLLECTION_STATUS(accountId)] || "idle";
            const currentFailCount = storageData[STORAGE_KEYS.FAIL_COUNT(accountId)] || 0;
            const lastCollectionTimestamp = storageData[STORAGE_KEYS.TIMESTAMP(accountId)] || null;
            const waitPeriod = storageData[STORAGE_KEYS.WAIT_PERIOD(accountId)] || 6;
            // Determine success based on status
            // If status is "success", it means a value update occurred
            const isSuccess = currentStatus === "success";
            this.log("info", `🔄 Queue processing: Completing collection for ${accountId} with ${isSuccess ? "success" : "failure"} (status: ${currentStatus}, failCount: ${currentFailCount})`);
            // Remove from active collections
            this.activeCollections.delete(accountId);
            // Prepare storage updates (to be applied in a single operation)
            const storageUpdates = {};
            // Always set status to idle
            storageUpdates[STORAGE_KEYS.COLLECTION_STATUS(accountId)] = "idle";
            this.statusCache.set(accountId, "idle");
            if (isSuccess) {
                // Collection was successful
                // Always update timestamp when collection succeeds
                // This timestamp is the source of truth for when a successful collection occurred
                storageUpdates[STORAGE_KEYS.TIMESTAMP(accountId)] =
                    new Date().toISOString();
                // Reset failCount to 0 if needed
                if (currentFailCount > 0) {
                    storageUpdates[STORAGE_KEYS.FAIL_COUNT(accountId)] = 0;
                    this.log("info", `✅ Collection succeeded for ${accountId}, resetting failCount to 0`);
                }
                this.log("info", `✅ Collection succeeded for ${accountId}, updating timestamp for future scheduling and failCount calculations`);
            }
            else {
                // Collection failed - check if we should increment failCount or reset it
                // Convert wait period to milliseconds
                const waitPeriodMs = waitPeriod * 60 * 60 * 1000;
                // Calculate time since last successful collection
                const now = Date.now();
                if (!lastCollectionTimestamp) {
                    // If we've never had a successful collection, always increment failCount
                    const newFailCount = currentFailCount + 1;
                    storageUpdates[STORAGE_KEYS.FAIL_COUNT(accountId)] = newFailCount;
                    this.log("info", `⚠️ No previous successful collection timestamp for ${accountId}, incrementing failCount to ${newFailCount}`);
                }
                else {
                    // Calculate time since last successful collection
                    const lastSuccessTime = typeof lastCollectionTimestamp === "string"
                        ? new Date(lastCollectionTimestamp).getTime()
                        : lastCollectionTimestamp;
                    const timeSinceLastSuccess = now - lastSuccessTime;
                    const timeSinceLastSuccessHours = Math.round(timeSinceLastSuccess / (60 * 60 * 1000));
                    if (timeSinceLastSuccess >= waitPeriodMs) {
                        // Enough time has passed since last successful collection, increment failCount
                        const newFailCount = currentFailCount + 1;
                        storageUpdates[STORAGE_KEYS.FAIL_COUNT(accountId)] = newFailCount;
                        this.log("info", `📊 Incrementing failCount for ${accountId} from ${currentFailCount} to ${newFailCount} (${timeSinceLastSuccessHours}h since last success, minimum required: ${waitPeriod}h)`);
                    }
                    else if (currentFailCount > 0) {
                        // Not enough time has passed AND failCount is positive - this is a false positive
                        // Reset the failCount to 0 as it shouldn't have been incremented
                        storageUpdates[STORAGE_KEYS.FAIL_COUNT(accountId)] = 0;
                        this.log("info", `🔄 Resetting false positive failCount for ${accountId} from ${currentFailCount} to 0: Only ${timeSinceLastSuccessHours}h since last collection (minimum required: ${waitPeriod}h)`);
                    }
                    else {
                        this.log("info", `ℹ️ Not incrementing failCount for ${accountId}: Only ${timeSinceLastSuccessHours}h since last successful collection (minimum required: ${waitPeriod}h)`);
                    }
                }
            }
            // Apply all storage updates in a single operation
            await chrome.storage.local.set(storageUpdates);
            // Update alarm based on the determined success status
            if (this.registry.storageAlarmManager) {
                await this.registry.storageAlarmManager.handleCollectionResult(accountId);
            }
        }
        catch (error) {
            this.log("error", `❌ Error completing collection for ${accountId}:`, error);
            this.activeCollections.delete(accountId);
            // Ensure we set the status to idle even if there's an error
            try {
                await this.setStatus(accountId, "idle");
            }
            catch (statusError) {
                this.log("error", `❌ Error resetting status for ${accountId}:`, statusError);
            }
        }
    }
    /**
     * Perform a health check to fix inconsistent states
     * Optimized to reduce storage operations
     */
    async performHealthCheck() {
        try {
            // Skip health check if no active collections and no recent queue activity
            if (this.activeCollections.size === 0 &&
                Date.now() - this.lastQueueFetch > this.HEALTH_CHECK_INTERVAL) {
                return;
            }
            // Get all collection statuses
            const storage = await chrome.storage.local.get(null);
            const statusPrefix = STORAGE_KEYS.COLLECTION_STATUS("").replace("_", "");
            const statusKeys = Object.keys(storage).filter((key) => key.startsWith(statusPrefix));
            // Count active collections
            const activeCount = this.activeCollections.size;
            if (activeCount > 0) {
                this.log("info", `🔍 Health check: ${activeCount} active collections`);
            }
            let orphanedCount = 0;
            let inconsistentCount = 0;
            const statusUpdates = {};
            // Fix orphaned collections
            for (const key of statusKeys) {
                const accountId = key.replace(statusPrefix, "");
                const status = storage[key];
                // Reset orphaned collections
                if (status === "collecting" && !this.activeCollections.has(accountId)) {
                    statusUpdates[key] = "idle";
                    this.statusCache.set(accountId, "idle");
                    orphanedCount++;
                }
                // Fix inconsistent active collections
                if (this.activeCollections.has(accountId) && status !== "collecting") {
                    inconsistentCount++;
                }
            }
            // Apply all status updates in a single operation
            if (Object.keys(statusUpdates).length > 0) {
                await chrome.storage.local.set(statusUpdates);
            }
            if (orphanedCount > 0 || inconsistentCount > 0) {
                this.log("info", `🔧 Health check: Fixed ${orphanedCount} orphaned and ${inconsistentCount} inconsistent collections`);
            }
        }
        catch (error) {
            this.log("error", "❌ Error during health check:", error);
        }
    }
    /**
     * Handle the queueProcessing alarm
     * This method can be called from the background script when the queueProcessing alarm fires
     */
    async handleQueueProcessingAlarm() {
        try {
            this.log("info", `⏰ Queue manager: Handling queueProcessing alarm`);
            // Process the queue
            await this.processQueue();
            // Run a health check
            await this.performHealthCheck();
        }
        catch (error) {
            this.log("error", `❌ Queue manager: Error handling queueProcessing alarm:`, error);
        }
    }
    /**
     * Get all collections with non-idle statuses
     * This is useful for debugging stuck collections
     */
    async getNonIdleCollections() {
        try {
            const storage = await chrome.storage.local.get(null);
            const result = {};
            // Find all collection status keys
            const statusKeys = Object.keys(storage).filter((key) => key.startsWith("collection_status_"));
            // Filter to only include non-idle statuses
            for (const key of statusKeys) {
                const status = storage[key];
                if (status !== "idle") {
                    const accountId = key.replace("collection_status_", "");
                    result[accountId] = status;
                    // Update cache
                    this.statusCache.set(accountId, status);
                }
            }
            return result;
        }
        catch (error) {
            this.log("error", `❌ Error getting non-idle collections:`, error);
            return {};
        }
    }
}

;// CONCATENATED MODULE: ./src/extension/managers/queue/index.ts


;// CONCATENATED MODULE: ./src/extension/managers/alarm/balanceLogger.ts
class BalanceLogger {
    static async logAllAccountBalances() {
        try {
            const { storedAccounts = {}, lastBalanceLog, customSites = [], } = (await chrome.storage.local.get([
                "storedAccounts",
                "lastBalanceLog",
                "customSites",
            ]));
            // If we have a lastBalanceLog, check if enough time has passed
            if (lastBalanceLog) {
                const now = new Date();
                const lastLog = new Date(lastBalanceLog);
                const timeSinceLastLog = now.getTime() - lastLog.getTime();
                const waitPeriodMs = 24 * 60 * 60 * 1000; // 24 hours in milliseconds
                if (timeSinceLastLog < waitPeriodMs) {
                    console.log("⏭️ Skipping balance log - Not enough time has passed");
                    return;
                }
            }
            // If we get here, either no lastBalanceLog exists or enough time has passed
            const now = new Date();
            // Process regular accounts
            for (const [accountId, account] of Object.entries(storedAccounts)) {
                await this.logAccountBalance(accountId, account, now);
            }
            // Process custom sites
            for (const account of customSites) {
                if (account.id) {
                    await this.logAccountBalance(account.id, account, now);
                }
            }
            // Update last balance log timestamp
            await chrome.storage.local.set({ lastBalanceLog: now.toISOString() });
            console.log("✅ Completed logging balances for active accounts");
        }
        catch (error) {
            console.error("❌ Error in logAllAccountBalances:", error);
        }
    }
    static async logAccountBalance(accountId, account, now) {
        try {
            // Check if account is active
            const { [`isActive_${accountId}`]: isActive } = await chrome.storage.local.get(`isActive_${accountId}`);
            // Skip if account is not active
            if (!isActive) {
                console.log(`⏭️ Skipping balance log for inactive account: ${accountId}`);
                return;
            }
            const balance = account?.currentVal || 0;
            const balanceEntry = {
                date: now.toISOString(),
                balance,
            };
            console.log(`📝 Logging balance for active account: ${accountId}`);
            const { [`balanceHistory_${accountId}`]: history = [] } = await chrome.storage.local.get(`balanceHistory_${accountId}`);
            history.push(balanceEntry);
            await chrome.storage.local.set({
                [`balanceHistory_${accountId}`]: history,
            });
        }
        catch (error) {
            console.error(`❌ Error logging balance for account ${accountId}:`, error);
        }
    }
}
/* harmony default export */ const balanceLogger = (BalanceLogger);

;// CONCATENATED MODULE: ./src/extension/managers/message/MessageManager.ts
/**
 * Manages message handling and routing for the extension
 */
class MessageManager {
    constructor(registry) {
        this.registry = registry;
    }
    /**
     * Handle a message from a content script or popup
     * @param message The message to handle
     * @param sender The sender of the message
     * @param sendResponse The function to call to send a response
     * @returns Whether to keep the message channel open
     */
    async handleMessage(message, sender, sendResponse) {
        try {
            // Extract message type
            const messageType = message.type || message.action;
            //console.log(`📨 Message received: ${messageType}`, message);
            // Route message to appropriate handler
            switch (messageType) {
                case "ACTION_STATUS":
                    return this.handleActionStatus(message, sender, sendResponse);
                case "PERFORM_COLLECTION":
                    return await this.handlePerformCollection(message, sendResponse);
                case "UPDATE_VALUE":
                    return await this.handleUpdateValue(message, sendResponse);
                case "QUEUE_ADD":
                    return this.handleQueueAdd(message, sendResponse);
                case "QUEUE_REMOVE":
                    return await this.handleQueueRemove(message, sendResponse);
                case "resetAllAlarms":
                    return await this.handleResetAllAlarms(sendResponse);
                case "RELAY_ACTION":
                    return this.handleRelayAction(message, sendResponse);
                case "UPDATE_TAB":
                    return this.handleUpdateTab(message, sendResponse);
                case "ADD_SITE_TO_QUEUE":
                    return this.handleAddSiteToQueue(message, sendResponse);
                case "UPDATE_ALARM":
                    return await this.handleUpdateAlarm(message, sendResponse);
                case "ACTION_RESULT":
                    return this.handleActionResult(message, sendResponse);
                default:
                    console.warn(`⚠️ Unhandled message type: ${messageType}`);
                    sendResponse({ success: false, error: "Unhandled message type" });
                    return false;
            }
        }
        catch (error) {
            console.error("❌ Error handling message:", error);
            sendResponse({ success: false, error: String(error) });
            return false;
        }
    }
    /**
     * Handle ACTION_STATUS messages
     */
    handleActionStatus(message, sender, sendResponse) {
        if (sender.tab?.id) {
            console.log(`✅ Action status update from tab ${sender.tab.id}:`, message);
            // Check if this is an OAuth completion message
            if (message.action === "googleOAuthComplete") {
                console.log(`🔑 Google OAuth completed for account: ${message.accountId}`);
            }
        }
        return false; // Don't keep message channel open
    }
    /**
     * Handle PERFORM_COLLECTION messages
     */
    async handlePerformCollection(message, sendResponse) {
        if (!message.data?.accountId || !message.data?.URL) {
            throw new Error("Missing required parameters for collection");
        }
        try {
            console.log(`🚀 Starting collection for ${message.data.accountId}`);
            await this.registry.collectionManager.performCollection(message.data.accountId, message.data.URL);
            console.log("✅ Collection completed for", message.data.accountId);
            sendResponse({ success: true });
        }
        catch (error) {
            console.error("❌ Collection error:", error);
            sendResponse({ success: false, error: String(error) });
        }
        return false; // Don't keep message channel open
    }
    /**
     * Handle UPDATE_VALUE messages
     */
    async handleUpdateValue(message, sendResponse) {
        if (!message.accountId || message.value === undefined) {
            throw new Error("Missing required parameters for UPDATE_VALUE");
        }
        try {
            // Use the centralized value update handler
            await this.registry.collectionManager.handleValueUpdate(message.accountId, message.value);
            // Send response back to the content script if possible
            try {
                sendResponse({ success: true });
            }
            catch (responseError) {
                // Silently ignore response errors - the content script might have closed
            }
        }
        catch (error) {
            console.error("❌ Value update error:", error);
            sendResponse({ success: false, error: String(error) });
        }
        return false; // Don't keep message channel open
    }
    /**
     * Handle QUEUE_ADD messages
     */
    handleQueueAdd(message, sendResponse) {
        const queueData = message.data || {
            accountId: message.accountId,
            URL: message.URL,
        };
        if (!queueData.accountId || !queueData.URL) {
            console.error("❌ Missing required parameters for queue add");
            sendResponse({
                success: false,
                error: "Missing required parameters for queue add",
            });
            return false; // No need to keep message channel open
        }
        try {
            console.log(`🔄 Adding ${queueData.accountId} to queue`);
            // Process the queue add asynchronously
            this.registry.backgroundQueueManager
                .addToQueue(queueData.accountId, queueData.URL)
                .then((success) => {
                console.log(`Queue result for ${queueData.accountId}: ${success ? "✅ Success" : "❌ Failed"}`);
                // Always send a response with clear success/failure indication
                sendResponse({
                    success,
                    error: success
                        ? null
                        : `Queue operation failed for ${queueData.accountId}`,
                });
            })
                .catch((error) => {
                console.error(`❌ Queue error for ${queueData.accountId}:`, error);
                sendResponse({
                    success: false,
                    error: String(error) ||
                        `Unknown error adding ${queueData.accountId} to queue`,
                });
            });
            return true; // Keep message channel open for async response
        }
        catch (error) {
            console.error(`❌ Queue error for ${queueData.accountId}:`, error);
            sendResponse({
                success: false,
                error: String(error) ||
                    `Unknown error adding ${queueData.accountId} to queue`,
            });
            return false; // Don't keep message channel open since we already responded
        }
    }
    /**
     * Handle QUEUE_REMOVE messages
     */
    async handleQueueRemove(message, sendResponse) {
        const accountId = message.data?.accountId || message.accountId;
        if (!accountId) {
            console.error("❌ Missing required parameter accountId for queue remove");
            sendResponse({
                success: false,
                error: "Missing required parameter accountId",
            });
            return true;
        }
        try {
            console.log(`🔄 Removing ${accountId} from queue`);
            const success = await this.registry.backgroundQueueManager.removeFromQueue(accountId);
            sendResponse({ success });
        }
        catch (error) {
            console.error(`❌ Queue remove error for ${accountId}:`, error);
            sendResponse({ success: false, error: String(error) });
        }
        return true;
    }
    /**
     * Handle resetAllAlarms messages
     */
    async handleResetAllAlarms(sendResponse) {
        console.log("🔄 Manual request to reset all alarms");
        try {
            await this.registry.resetAllAlarms();
            sendResponse({
                success: true,
                message: "All alarms have been reset",
            });
        }
        catch (error) {
            console.error("❌ Error resetting alarms:", error);
            sendResponse({ success: false, error: String(error) });
        }
        return false; // Don't keep message channel open
    }
    /**
     * Handle RELAY_ACTION messages
     */
    handleRelayAction(message, sendResponse) {
        // Extract tabId and action from the message
        const { tabId, action } = message;
        if (!tabId || !action) {
            console.error("❌ Missing tabId or action in RELAY_ACTION message");
            sendResponse({ success: false, error: "Missing tabId or action" });
            return true;
        }
        console.log(`🔄 Relaying action ${action.action} to tab ${tabId}`);
        // Special handling for ping actions - they need to be fast and reliable
        if (action.action === "ping") {
            try {
                chrome.tabs.sendMessage(tabId, { action: "ping" }, (response) => {
                    if (chrome.runtime.lastError) {
                        console.warn("Ping relay error:", chrome.runtime.lastError.message);
                        sendResponse({
                            success: false,
                            error: chrome.runtime.lastError.message,
                        });
                    }
                    else {
                        sendResponse(response || { status: "alive", success: true });
                    }
                });
            }
            catch (error) {
                console.error("❌ Error relaying ping:", error);
                sendResponse({ success: false, error: String(error) });
            }
            return true;
        }
        // Keep the message channel open
        return true;
    }
    /**
     * Handle ACTION_RESULT messages
     */
    handleActionResult(message, sendResponse) {
        console.log("🔄 Action result:", message);
        // Forward action result to collection manager
        // try {
        //   if (this.registry.collectionManager) {
        //     this.registry.collectionManager.handleActionResult(message);
        //     sendResponse({ success: true });
        //   } else {
        //     console.warn("⚠️ Collection manager not available for ACTION_RESULT");
        //     sendResponse({
        //       success: false,
        //       error: "Collection manager not available",
        //     });
        //   }
        // } catch (error) {
        //   console.error("❌ Error handling action result:", error);
        //   sendResponse({ success: false, error: String(error) });
        // }
        return true;
    }
    /**
     * Handle UPDATE_TAB messages
     */
    handleUpdateTab(message, sendResponse) {
        // Extract tabId and properties from the message
        const { tabId, properties } = message;
        if (!tabId || !properties) {
            console.error("❌ Missing tabId or properties in UPDATE_TAB message");
            sendResponse({
                success: false,
                error: "Missing tabId or properties",
            });
            return true;
        }
        console.log(`🔄 Updating tab ${tabId} with properties:`, properties);
        // Update the tab
        try {
            chrome.tabs.update(tabId, properties, (tab) => {
                if (chrome.runtime.lastError) {
                    console.warn("Tab update error:", chrome.runtime.lastError);
                    sendResponse({
                        success: false,
                        error: chrome.runtime.lastError.message,
                    });
                    return;
                }
                // Send back the updated tab
                sendResponse({ success: true, tab });
            });
        }
        catch (error) {
            console.error("❌ Error updating tab:", error);
            sendResponse({ success: false, error: String(error) });
        }
        // Keep the message channel open
        return true;
    }
    /**
     * Handle ADD_SITE_TO_QUEUE messages
     */
    handleAddSiteToQueue(message, sendResponse) {
        const { accountId, siteType } = message;
        if (!accountId || !siteType) {
            console.error("❌ Missing required parameters for ADD_SITE_TO_QUEUE");
            sendResponse({
                success: false,
                error: "Missing required parameters",
            });
            return true;
        }
        // Since we've removed site configuration, we'll just return an error
        console.warn("⚠️ Site configuration has been removed from the system");
        sendResponse({
            success: false,
            error: "Site configuration functionality has been removed",
        });
        return true;
    }
    /**
     * Handle UPDATE_ALARM messages
     * This is used to directly update the alarm for an account when a value update is successful
     */
    async handleUpdateAlarm(message, sendResponse) {
        const { accountId, success } = message;
        if (!accountId) {
            console.error("❌ Missing required parameter accountId for UPDATE_ALARM");
            sendResponse({
                success: false,
                error: "Missing required parameter accountId",
            });
            return false;
        }
        try {
            console.log(`⏰ Handling direct alarm update for ${accountId}, success: ${success}`);
            if (this.registry.storageAlarmManager) {
                await this.registry.storageAlarmManager.handleCollectionResult(accountId, success);
                console.log(`✅ Successfully updated alarm for ${accountId}`);
                sendResponse({ success: true });
            }
            else {
                console.warn("⚠️ Storage alarm manager not available");
                sendResponse({
                    success: false,
                    error: "Storage alarm manager not available",
                });
            }
        }
        catch (error) {
            console.error(`❌ Error updating alarm for ${accountId}:`, error);
            sendResponse({ success: false, error: String(error) });
        }
        return false; // Don't keep message channel open
    }
}

;// CONCATENATED MODULE: ./src/extension/background.ts



// Import the BackgroundQueueManager from its new location





// Export the ManagerRegistry class
class ManagerRegistry {
    constructor() {
        this._alarmManager = null;
        this._backgroundQueueManager = null;
        this._collectionManager = null;
        this._badgeManager = null;
        this.initializationPromise = null;
        this.recentValueUpdates = new Map();
        this.recentSuccessLogs = new Map();
    }
    // Add methods to manage recent updates cache
    hasRecentValueUpdate(key) {
        return this.recentValueUpdates.has(key);
    }
    getRecentValueUpdateTime(key) {
        return this.recentValueUpdates.get(key);
    }
    setRecentValueUpdate(key, time) {
        this.recentValueUpdates.set(key, time);
    }
    // Add methods to manage recent success logs cache
    hasRecentSuccessLog(key) {
        return this.recentSuccessLogs.has(key);
    }
    getRecentSuccessLogTime(key) {
        return this.recentSuccessLogs.get(key);
    }
    setRecentSuccessLog(key, time) {
        this.recentSuccessLogs.set(key, time);
    }
    static getInstance() {
        if (!ManagerRegistry.instance) {
            ManagerRegistry.instance = new ManagerRegistry();
        }
        return ManagerRegistry.instance;
    }
    async initialize(forceInit = false) {
        if (this.initializationPromise) {
            return this.initializationPromise;
        }
        this.initializationPromise = this._initialize(forceInit);
        return this.initializationPromise;
    }
    async _initialize(forceInit) {
        try {
            console.log("🚀 Initializing ManagerRegistry...");
            // Always initialize badge manager first for real-time updates
            if (!this._badgeManager || forceInit) {
                console.log("📊 Initializing BadgeManager");
                await this.initializeBadgeManager();
            }
            // Initialize other managers
            if (!this._alarmManager || forceInit) {
                console.log("⏰ Initializing AlarmManager");
                this._alarmManager = new StorageAlarmManager();
            }
            if (!this._backgroundQueueManager || forceInit) {
                console.log("🔄 Initializing QueueManager");
                this._backgroundQueueManager = new BackgroundQueueManager(this);
            }
            if (!this._collectionManager || forceInit) {
                console.log("📥 Initializing CollectionManager");
                this._collectionManager = new CollectionManager();
            }
            if (forceInit) {
                console.log("🧹 Cleaning up stale states");
                await this.cleanupStaleStates();
                // Reset all collection statuses to idle on extension refresh
                if (this._collectionManager) {
                    console.log("🔄 Resetting all collection statuses to idle");
                    await this._collectionManager.resetAllCollectionStatuses();
                }
                // Explicitly reset the queue on extension refresh
                if (this._backgroundQueueManager) {
                    console.log("🔄 Explicitly resetting queue on extension refresh");
                    await this._backgroundQueueManager.resetQueue();
                }
                console.log("⏰ Setting up alarms for all accounts");
                await this._alarmManager.checkAndSetAlarmsForAllAccounts();
                console.log("💓 Setting up keep-alive");
                await this.setupKeepAlive();
            }
            // Always update badge
            await BadgeManager.updateBadge();
            console.log("✅ ManagerRegistry initialization complete");
        }
        catch (error) {
            console.error("❌ Error in manager initialization:", error);
            this.initializationPromise = null;
            throw error;
        }
    }
    async cleanupStaleStates() {
        try {
            console.log("🧹 Cleaning up stale states...");
            // Get all storage data in one operation
            const allStorage = await chrome.storage.local.get(null);
            // Prepare updates and keys to remove
            const updates = {};
            const keysToRemove = [];
            // Find all account IDs
            const accountIds = Object.keys(allStorage)
                .filter((key) => key.startsWith("isActive_"))
                .map((key) => key.replace("isActive_", ""));
            // Process all accounts in one pass
            for (const accountId of accountIds) {
                // Reset collection status if not idle
                const statusKey = STORAGE_KEYS.COLLECTION_STATUS(accountId);
                const status = allStorage[statusKey];
                if (status && status !== "idle") {
                    updates[statusKey] = "idle";
                }
            }
            // Find all lock keys in one pass
            const lockKeys = Object.keys(allStorage).filter((key) => key.startsWith("collection_lock_") || key === "queue_processing_lock");
            keysToRemove.push(...lockKeys);
            // Add queue-related keys to remove
            const queueKeysToRemove = [
                "collection_queue",
                "queue_items",
                "current_collection",
            ];
            keysToRemove.push(...queueKeysToRemove);
            // Execute all storage operations in parallel
            if (Object.keys(updates).length > 0) {
                await chrome.storage.local.set(updates);
            }
            if (keysToRemove.length > 0) {
                await chrome.storage.local.remove(keysToRemove);
            }
            console.log("✅ Stale states cleanup completed");
        }
        catch (error) {
            console.error("❌ Error cleaning up stale states:", error);
        }
    }
    async setupKeepAlive() {
        try {
            console.log("🔄 Setting up keep-alive systems...");
            // Clear all alarms in one operation
            const alarmsToRemove = [
                "systemCheck",
                ALARM_TYPES.BALANCE_LOGGER,
                "queueHealthCheck",
            ];
            await Promise.all(alarmsToRemove.map((name) => chrome.alarms.clear(name)));
            // Initialize all managers and setup checks in parallel
            await Promise.all([
                this.initializeBadgeManager(),
                this.setupSystemCheck(),
                this.setupBalanceLogger(),
                this.setupQueueHealthCheck(),
                this.verifyAlarms(),
            ]);
            console.log("✅ Keep-alive systems initialized");
        }
        catch (error) {
            console.error("❌ Error setting up keep-alive:", error);
        }
    }
    async initializeBadgeManager() {
        if (!this._badgeManager) {
            await BadgeManager.initialize();
            await BadgeManager.updateBadge();
            this._badgeManager = BadgeManager;
        }
    }
    async setupSystemCheck() {
        await chrome.alarms.create("systemCheck", {
            periodInMinutes: 2,
            when: Date.now() + 1000,
        });
    }
    async setupBalanceLogger() {
        // Run balance logger every hour
        await chrome.alarms.create(ALARM_TYPES.BALANCE_LOGGER, {
            periodInMinutes: 60,
            when: Date.now() + 1000,
        });
    }
    async setupQueueHealthCheck() {
        await chrome.alarms.create("queueHealthCheck", {
            periodInMinutes: 5, // Run every 5 minutes
            when: Date.now() + 1000,
        });
    }
    /**
     * Reset all alarms for all accounts
     * This is useful for testing or when alarms are suspected to be incorrect
     */
    async resetAllAlarms() {
        try {
            console.log("🔄 Starting complete alarm reset for all accounts");
            const alarmManager = this._alarmManager;
            if (!alarmManager) {
                console.error("❌ Alarm manager not initialized");
                return;
            }
            await alarmManager.checkAndSetAlarmsForAllAccounts();
        }
        catch (error) {
            console.error("❌ Error resetting alarms:", error);
        }
    }
    /**
     * Verify alarms for all accounts
     * This method is called during service worker initialization
     */
    async verifyAlarms() {
        try {
            console.log("🔍 Starting comprehensive alarm verification for all accounts");
            // First clean up any invalid account alarms
            await cleanupInvalidAccountAlarms();
            const alarmManager = this._alarmManager;
            if (!alarmManager) {
                console.error("❌ Alarm manager not initialized");
                return;
            }
            await alarmManager.checkAndSetAlarmsForAllAccounts();
            // Get alarms again after validation to see what changed
            const alarmsAfter = await chrome.alarms.getAll();
            const collectionAlarmsAfter = alarmsAfter.filter((alarm) => alarm.name.startsWith(ALARM_TYPES.COLLECTION));
            console.log(`After validation: ${collectionAlarmsAfter.length} collection alarms out of ${alarmsAfter.length} total alarms`);
            console.log("✅ Comprehensive alarm verification completed");
        }
        catch (error) {
            console.error("❌ Error verifying alarms:", error);
        }
    }
    // Getters with lazy initialization
    get alarmManager() {
        if (!this._alarmManager) {
            throw new Error("AlarmManager not initialized");
        }
        return this._alarmManager;
    }
    get backgroundQueueManager() {
        if (!this._backgroundQueueManager) {
            this._backgroundQueueManager = new BackgroundQueueManager(this);
        }
        return this._backgroundQueueManager;
    }
    get collectionManager() {
        if (!this._collectionManager) {
            this._collectionManager = new CollectionManager();
        }
        return this._collectionManager;
    }
    get badgeManager() {
        if (!this._badgeManager) {
            throw new Error("BadgeManager not initialized");
        }
        return this._badgeManager;
    }
    get storageAlarmManager() {
        if (!this._alarmManager) {
            throw new Error("StorageAlarmManager not initialized");
        }
        return this._alarmManager;
    }
}
// Replace global managers with registry
const managerRegistry = ManagerRegistry.getInstance();
let messageManager = null;
/**
 * Clean up alarms for accounts that are paused or don't have UserKeys
 */
async function cleanupInvalidAccountAlarms() {
    try {
        console.log("🧹 Cleaning up alarms for paused or invalid accounts...");
        // Get all alarms
        const alarms = await chrome.alarms.getAll();
        const collectionAlarms = alarms.filter((alarm) => alarm.name.startsWith(ALARM_TYPES.COLLECTION));
        console.log(`🔍 Found ${collectionAlarms.length} collection alarms to check`);
        // Track statistics
        let removedCount = 0;
        let manualAlarmsPreserved = 0;
        // Process each collection alarm
        for (const alarm of collectionAlarms) {
            const accountId = alarm.name.replace(ALARM_TYPES.COLLECTION, "");
            // Check if account is paused
            const pausedKey = STORAGE_KEYS.PAUSED(accountId);
            const { [pausedKey]: isPaused } = await chrome.storage.local.get(pausedKey);
            // Check if account has UserKeys
            const userKeysKey = `UserKeys_${accountId}`;
            const { [userKeysKey]: userKeys } = await chrome.storage.local.get(userKeysKey);
            // Check if account is active
            const isActiveKey = STORAGE_KEYS.IS_ACTIVE(accountId);
            const { [isActiveKey]: isActive } = await chrome.storage.local.get(isActiveKey);
            // Check if it's a manual alarm
            const manualAlarmKey = STORAGE_KEYS.MANUAL_ALARM(accountId);
            const { [manualAlarmKey]: isManualAlarmSet = false } = await chrome.storage.local.get(manualAlarmKey);
            // Check the alarm status to confirm it's a manual alarm
            const alarmStatusKey = STORAGE_KEYS.ALARM_STATUS(accountId);
            const { [alarmStatusKey]: alarmStatus } = await chrome.storage.local.get(alarmStatusKey);
            const isManualAlarmType = alarmStatus && alarmStatus.type === "manual";
            // If it's a manual alarm, preserve it regardless of other conditions
            if (isManualAlarmSet || isManualAlarmType) {
                console.log(`🔒 Preserving manual alarm for ${accountId}`);
                manualAlarmsPreserved++;
                continue;
            }
            // If account is paused or doesn't have UserKeys or is explicitly set as inactive, remove the alarm
            if (isPaused || !userKeys || isActive === false) {
                const reason = isPaused
                    ? "paused"
                    : !userKeys
                        ? "missing UserKeys"
                        : "inactive";
                console.log(`🗑️ Removing alarm for ${accountId} (${reason})`);
                await chrome.alarms.clear(alarm.name);
                removedCount++;
            }
        }
        console.log(`✅ Cleanup completed: Removed ${removedCount} alarms for paused or invalid accounts, preserved ${manualAlarmsPreserved} manual alarms`);
    }
    catch (error) {
        console.error("❌ Error cleaning up invalid account alarms:", error);
    }
}
// Initialize the ManagerRegistry when the service worker starts
(async function initializeServiceWorker() {
    try {
        console.log("🚀 Service worker starting - initializing systems");
        // Initialize the manager registry
        await managerRegistry.initialize();
        // Initialize the message manager
        messageManager = new MessageManager(managerRegistry);
        console.log("✅ Service worker initialization complete");
        // Set up recurring alarms
        await setupRecurringAlarms();
        // Clean up alarms for paused or invalid accounts
        await cleanupInvalidAccountAlarms();
        // Verify alarms for all accounts
        await managerRegistry.verifyAlarms();
        // Update badge
        await updateBadgeAfterInitialization();
        console.log("✅ Service worker fully initialized");
    }
    catch (error) {
        console.error("❌ Error initializing service worker:", error);
    }
})();
/**
 * Update badge after initialization
 */
async function updateBadgeAfterInitialization() {
    try {
        await BadgeManager.updateBadge();
        console.log("✅ Badge updated after initialization");
    }
    catch (error) {
        console.error("❌ Error updating badge:", error);
    }
}
// Handle extension installation or update
chrome.runtime.onInstalled.addListener(async (details) => {
    await handleExtensionInstallOrUpdate(details);
});
// Handle extension startup (when browser starts or extension is refreshed)
chrome.runtime.onStartup.addListener(async () => {
    await handleExtensionStartup();
});
// Update message listener to use MessageManager
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    // Ensure message manager is initialized
    if (!messageManager) {
        console.error("❌ Message manager not initialized");
        sendResponse({ success: false, error: "Message manager not initialized" });
        return false;
    }
    // Delegate to message manager
    return messageManager.handleMessage(message, sender, sendResponse);
});
/**
 * Handle extension installation or update
 */
async function handleExtensionInstallOrUpdate(details) {
    try {
        console.log("🔄 Extension installed/updated:", details.reason);
        // Force re-initialization
        await managerRegistry.initialize(true);
        // Verify alarms after initialization
        await managerRegistry.verifyAlarms();
        console.log("✅ Extension initialization complete after install/update");
    }
    catch (error) {
        console.error("❌ Error initializing after install/update:", error);
    }
}
/**
 * Handle extension startup
 */
async function handleExtensionStartup() {
    try {
        console.log("🔄 Extension starting up - resetting collection statuses");
        // Initialize the manager registry
        await managerRegistry.initialize();
        // Reset all collection statuses to idle
        if (managerRegistry.collectionManager) {
            await managerRegistry.collectionManager.resetAllCollectionStatuses();
            console.log("✅ All collection statuses reset to idle on startup");
        }
        // Clean up alarms for paused or invalid accounts
        await cleanupInvalidAccountAlarms();
        // Verify alarms after reset
        await managerRegistry.verifyAlarms();
    }
    catch (error) {
        console.error("❌ Error during startup reset:", error);
    }
}
// Set up recurring alarms
async function setupRecurringAlarms() {
    try {
        // Hourly check alarm - check every hour that all accounts have alarms
        await chrome.alarms.create(ALARM_TYPES.HOURLY_CHECK, {
            periodInMinutes: 60,
        });
        console.log("⏰ Set up hourly check alarm");
        // Resolve alarm - check every 3 hours for accounts that need resolving
        await chrome.alarms.create(ALARM_TYPES.RESOLVE, {
            periodInMinutes: 180,
        });
        console.log("⏰ Set up resolve alarm");
        // Keepalive alarm - ping every 25 minutes to keep service worker alive
        await chrome.alarms.create(ALARM_TYPES.KEEPALIVE, {
            periodInMinutes: 25,
        });
        console.log("⏰ Set up keepalive alarm");
        // Balance logger alarm - log account balances every hour
        await chrome.alarms.create(ALARM_TYPES.BALANCE_LOGGER, {
            periodInMinutes: 60,
            when: Date.now() + 1000,
        });
        console.log("⏰ Set up balance logger alarm");
    }
    catch (error) {
        console.error("❌ Error setting up recurring alarms:", error);
    }
}
/**
 * Handle collection alarm for a specific account
 */
async function handleCollectionAlarm(accountId) {
    try {
        console.log(`🔄 Handling collection alarm for ${accountId}`);
        // Check if the account is paused
        const pausedKey = STORAGE_KEYS.PAUSED(accountId);
        const { [pausedKey]: isPaused } = await chrome.storage.local.get(pausedKey);
        if (isPaused) {
            console.log(`⏸️ Skipping collection for paused account: ${accountId}`);
            return;
        }
        // Get the account URL
        const accountData = await getAccountData(accountId);
        // Check if it's a custom site
        const isCustomSite = await isCustomSiteAccount(accountId);
        if (isCustomSite && accountData.URL) {
            // For custom sites, check if there's already a tab with this URL open
            // to prevent duplicate tabs from being created
            const allTabs = await chrome.tabs.query({});
            const normalizedUrl = new URL(accountData.URL).toString();
            const existingTab = allTabs.find((tab) => {
                if (tab.url) {
                    try {
                        // Compare URL hostnames and paths to check if it's the same site
                        const tabUrl = new URL(tab.url);
                        const accountUrl = new URL(normalizedUrl);
                        return (tabUrl.hostname === accountUrl.hostname &&
                            tabUrl.pathname === accountUrl.pathname);
                    }
                    catch (e) {
                        return false;
                    }
                }
                return false;
            });
            if (existingTab && existingTab.id) {
                console.log(`🔄 Found existing tab (${existingTab.id}) for URL: ${accountData.URL}, using it instead of creating new one`);
                // Optionally refresh the tab to ensure content is up to date
                await chrome.tabs.reload(existingTab.id);
            }
            else {
                // No existing tab found, create a new one
                console.log(`🌐 Opening custom site URL in background tab: ${accountData.URL}`);
                // Create tab in background
                await chrome.tabs.create({ url: accountData.URL, active: false });
            }
        }
        else {
            // For regular accounts, add to queue for collection
            await addAccountToCollectionQueue(accountId, accountData.URL);
        }
    }
    catch (error) {
        console.error(`❌ Error handling collection alarm for ${accountId}:`, error);
    }
}
/**
 * Add account to collection queue
 */
async function addAccountToCollectionQueue(accountId, url) {
    try {
        const success = await managerRegistry.backgroundQueueManager.addToQueue(accountId, url);
        if (success) {
            console.log(`✅ Added ${accountId} to collection queue`);
        }
        else {
            console.error(`❌ Failed to add ${accountId} to collection queue`);
        }
    }
    catch (error) {
        console.error(`❌ Error adding ${accountId} to collection queue:`, error);
    }
}
// Replace global alarms with registry
chrome.alarms.onAlarm.addListener(async (alarm) => {
    try {
        console.log(`⏰ Alarm triggered: ${alarm.name}`);
        await handleAlarm(alarm);
    }
    catch (error) {
        console.error("❌ Error handling alarm:", error);
    }
});
/**
 * Central alarm handler that delegates to appropriate handlers based on alarm type
 */
async function handleAlarm(alarm) {
    const alarmName = alarm.name;
    // Handle collection alarms
    if (alarmName.startsWith(ALARM_TYPES.COLLECTION)) {
        const accountId = alarmName.replace(ALARM_TYPES.COLLECTION, "");
        console.log(`🔄 Processing collection alarm for ${accountId}`);
        await handleCollectionAlarm(accountId);
        return;
    }
    // Handle system alarms
    switch (alarmName) {
        case ALARM_TYPES.HOURLY_CHECK:
            console.log("🔄 Processing hourly check alarm");
            await managerRegistry.verifyAlarms();
            break;
        case ALARM_TYPES.RESOLVE:
            console.log("🔄 Processing resolve alarm");
            // Resolve alarm functionality has been removed
            break;
        case ALARM_TYPES.KEEPALIVE:
            console.log("🔄 Processing keepalive alarm");
            // Keepalive logic here
            break;
        case ALARM_TYPES.BALANCE_LOGGER:
            console.log("📊 Processing balance logger alarm");
            try {
                await balanceLogger.logAllAccountBalances();
                console.log("✅ Balance logging completed");
            }
            catch (error) {
                console.error("❌ Error in balance logging:", error);
            }
            break;
        case "systemCheck":
            // console.log("🔄 Processing system check alarm");
            // await managerRegistry.verifyAlarms();
            break;
        case "queueHealthCheck":
            console.log("🔄 Processing queue health check alarm");
            // Check if queue is stuck and reset if needed
            try {
                // Check if there are any active collections
                const queueManager = managerRegistry.backgroundQueueManager;
                if (queueManager) {
                    // Just log the queue status for now
                    console.log("📊 Queue health check completed");
                }
            }
            catch (error) {
                console.error("❌ Error in queue health check:", error);
            }
            break;
        case "queueProcessing":
            console.log("🔄 Processing queueProcessing alarm");
            try {
                const queueManager = managerRegistry.backgroundQueueManager;
                if (queueManager) {
                    await queueManager.handleQueueProcessingAlarm();
                    console.log("✅ Queue processing alarm handled");
                }
                else {
                    console.error("❌ Queue manager not available for queueProcessing alarm");
                }
            }
            catch (error) {
                console.error("❌ Error handling queueProcessing alarm:", error);
            }
            break;
        default:
            console.log(`⚠️ Unhandled alarm type: ${alarmName}`);
    }
}
/**
 * Get account data from storage
 * @param accountId The account ID
 * @returns The account data including URL
 */
async function getAccountData(accountId) {
    try {
        const { storedAccounts = {}, customSites = [] } = await chrome.storage.local.get(["storedAccounts", "customSites"]);
        // First check regular stored accounts
        if (storedAccounts[accountId]) {
            return storedAccounts[accountId];
        }
        // Then check custom sites
        const customSite = customSites.find((site) => site.id === accountId);
        if (customSite) {
            return customSite;
        }
        // Return empty object if not found in either
        return { URL: "" };
    }
    catch (error) {
        console.error(`❌ Error getting account data for ${accountId}:`, error);
        return { URL: "" };
    }
}
/**
 * Check if an account ID belongs to a custom site
 * @param accountId The account ID to check
 * @returns Promise that resolves to true if it's a custom site, false otherwise
 */
async function isCustomSiteAccount(accountId) {
    try {
        const { customSites = [] } = await chrome.storage.local.get("customSites");
        return customSites.some((site) => site.id === accountId);
    }
    catch (error) {
        console.error(`❌ Error checking if ${accountId} is a custom site:`, error);
        return false;
    }
}

/******/ })()
;
//# sourceMappingURL=background.js.map